# Copyright 2023 LiveKit, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""LiveKit SDK for Python
`pip install livekit`

See https://docs.livekit.io/home/client/connect/#installing-the-livekit-sdk for more information.
"""

from ._proto import stats_pb2 as stats
from ._proto.e2ee_pb2 import EncryptionState, EncryptionType
from ._proto.participant_pb2 import ParticipantKind, DisconnectReason
from ._proto.room_pb2 import (
    ConnectionQuality,
    ConnectionState,
    ContinualGatheringPolicy,
    DataPacketKind,
    IceServer,
    IceTransportType,
    TrackPublishOptions,
    VideoEncoding,
)
from ._proto.track_pb2 import (
    StreamState,
    TrackKind,
    TrackSource,
    ParticipantTrackPermission,
)
from ._proto.video_frame_pb2 import VideoBufferType, VideoCodec, VideoRotation
from .audio_frame import AudioFrame
from .audio_source import AudioSource
from .audio_stream import AudioFrameEvent, AudioStream, NoiseCancellationOptions
from .audio_filter import AudioFilter
from .e2ee import (
    E2EEManager,
    E2EEOptions,
    FrameCryptor,
    KeyProvider,
    KeyProviderOptions,
)
from .participant import (
    LocalParticipant,
    Participant,
    RemoteParticipant,
)
from .room import (
    ConnectError,
    DataPacket,
    Room,
    RoomOptions,
    RtcConfiguration,
    SipDTMF,
    RtcStats,
)
from .track import (
    AudioTrack,
    LocalAudioTrack,
    LocalTrack,
    LocalVideoTrack,
    RemoteAudioTrack,
    RemoteTrack,
    RemoteVideoTrack,
    Track,
    VideoTrack,
)
from .event_emitter import EventEmitter
from .track_publication import (
    LocalTrackPublication,
    RemoteTrackPublication,
    TrackPublication,
)
from .transcription import Transcription, TranscriptionSegment
from .version import __version__
from .video_frame import (
    VideoFrame,
)
from .video_source import VideoSource
from .video_stream import VideoFrameEvent, VideoStream
from .audio_resampler import AudioResampler, AudioResamplerQuality
from .audio_mixer import AudioMixer
from .apm import AudioProcessingModule
from .utils import combine_audio_frames
from .rpc import RpcError, RpcInvocationData
from .synchronizer import AVSynchronizer
from .data_stream import (
    TextStreamInfo,
    ByteStreamInfo,
    TextStreamReader,
    TextStreamWriter,
    ByteStreamWriter,
    ByteStreamReader,
)
from .recorder import (
    ParticipantRecorder,
    RecordingStats,
    RecordingError,
    ParticipantNotFoundError,
    TrackNotFoundError,
    WebMEncoderNotAvailableError,
)

__all__ = [
    "ConnectionQuality",
    "ConnectionState",
    "DataPacketKind",
    "TrackPublishOptions",
    "IceTransportType",
    "ContinualGatheringPolicy",
    "IceServer",
    "EncryptionType",
    "EncryptionState",
    "StreamState",
    "TrackKind",
    "TrackSource",
    "ParticipantTrackPermission",
    "VideoBufferType",
    "VideoRotation",
    "stats",
    "AudioFrame",
    "AudioSource",
    "AudioStream",
    "NoiseCancellationOptions",
    "AudioFilter",
    "AudioFrameEvent",
    "LocalParticipant",
    "Participant",
    "ParticipantKind",
    "DisconnectReason",
    "RemoteParticipant",
    "ConnectError",
    "Room",
    "RoomOptions",
    "RtcConfiguration",
    "SipDTMF",
    "RtcStats",
    "DataPacket",
    "LocalAudioTrack",
    "LocalVideoTrack",
    "RemoteAudioTrack",
    "RemoteVideoTrack",
    "Track",
    "LocalTrack",
    "RemoteTrack",
    "AudioTrack",
    "VideoTrack",
    "E2EEManager",
    "E2EEOptions",
    "KeyProviderOptions",
    "KeyProvider",
    "FrameCryptor",
    "LocalTrackPublication",
    "RemoteTrackPublication",
    "TrackPublication",
    "Transcription",
    "TranscriptionSegment",
    "VideoCodec",
    "VideoEncoding",
    "VideoFrame",
    "VideoFrameEvent",
    "VideoSource",
    "VideoStream",
    "AudioMixer",
    "AudioResampler",
    "AudioResamplerQuality",
    "RpcError",
    "RpcInvocationData",
    "EventEmitter",
    "combine_audio_frames",
    "AVSynchronizer",
    "TextStreamInfo",
    "ByteStreamInfo",
    "TextStreamReader",
    "TextStreamWriter",
    "ByteStreamReader",
    "ByteStreamWriter",
    "AudioProcessingModule",
    "ParticipantRecorder",
    "RecordingStats",
    "RecordingError",
    "ParticipantNotFoundError",
    "TrackNotFoundError",
    "WebMEncoderNotAvailableError",
    "__version__",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stats.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0bstats.proto\x12\rlivekit.proto\"\xeb\x18\n\x08RtcStats\x12.\n\x05\x63odec\x18\x03 \x01(\x0b\x32\x1d.livekit.proto.RtcStats.CodecH\x00\x12\x39\n\x0binbound_rtp\x18\x04 \x01(\x0b\x32\".livekit.proto.RtcStats.InboundRtpH\x00\x12;\n\x0coutbound_rtp\x18\x05 \x01(\x0b\x32#.livekit.proto.RtcStats.OutboundRtpH\x00\x12\x46\n\x12remote_inbound_rtp\x18\x06 \x01(\x0b\x32(.livekit.proto.RtcStats.RemoteInboundRtpH\x00\x12H\n\x13remote_outbound_rtp\x18\x07 \x01(\x0b\x32).livekit.proto.RtcStats.RemoteOutboundRtpH\x00\x12;\n\x0cmedia_source\x18\x08 \x01(\x0b\x32#.livekit.proto.RtcStats.MediaSourceH\x00\x12=\n\rmedia_playout\x18\t \x01(\x0b\x32$.livekit.proto.RtcStats.MediaPlayoutH\x00\x12\x41\n\x0fpeer_connection\x18\n \x01(\x0b\x32&.livekit.proto.RtcStats.PeerConnectionH\x00\x12;\n\x0c\x64\x61ta_channel\x18\x0b \x01(\x0b\x32#.livekit.proto.RtcStats.DataChannelH\x00\x12\x36\n\ttransport\x18\x0c \x01(\x0b\x32!.livekit.proto.RtcStats.TransportH\x00\x12?\n\x0e\x63\x61ndidate_pair\x18\r \x01(\x0b\x32%.livekit.proto.RtcStats.CandidatePairH\x00\x12\x41\n\x0flocal_candidate\x18\x0e \x01(\x0b\x32&.livekit.proto.RtcStats.LocalCandidateH\x00\x12\x43\n\x10remote_candidate\x18\x0f \x01(\x0b\x32\'.livekit.proto.RtcStats.RemoteCandidateH\x00\x12:\n\x0b\x63\x65rtificate\x18\x10 \x01(\x0b\x32#.livekit.proto.RtcStats.CertificateH\x00\x12\x30\n\x06stream\x18\x11 \x01(\x0b\x32\x1e.livekit.proto.RtcStats.StreamH\x00\x12.\n\x05track\x18\x12 \x01(\x0b\x32\x1d.livekit.proto.RtcStats.TrackH\x00\x1a[\n\x05\x43odec\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12(\n\x05\x63odec\x18\x02 \x02(\x0b\x32\x19.livekit.proto.CodecStats\x1a\xd5\x01\n\nInboundRtp\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12-\n\x06stream\x18\x02 \x02(\x0b\x32\x1d.livekit.proto.RtpStreamStats\x12\x37\n\x08received\x18\x03 \x02(\x0b\x32%.livekit.proto.ReceivedRtpStreamStats\x12\x35\n\x07inbound\x18\x04 \x02(\x0b\x32$.livekit.proto.InboundRtpStreamStats\x1a\xd0\x01\n\x0bOutboundRtp\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12-\n\x06stream\x18\x02 \x02(\x0b\x32\x1d.livekit.proto.RtpStreamStats\x12/\n\x04sent\x18\x03 \x02(\x0b\x32!.livekit.proto.SentRtpStreamStats\x12\x37\n\x08outbound\x18\x04 \x02(\x0b\x32%.livekit.proto.OutboundRtpStreamStats\x1a\xe8\x01\n\x10RemoteInboundRtp\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12-\n\x06stream\x18\x02 \x02(\x0b\x32\x1d.livekit.proto.RtpStreamStats\x12\x37\n\x08received\x18\x03 \x02(\x0b\x32%.livekit.proto.ReceivedRtpStreamStats\x12\x42\n\x0eremote_inbound\x18\x04 \x02(\x0b\x32*.livekit.proto.RemoteInboundRtpStreamStats\x1a\xe3\x01\n\x11RemoteOutboundRtp\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12-\n\x06stream\x18\x02 \x02(\x0b\x32\x1d.livekit.proto.RtpStreamStats\x12/\n\x04sent\x18\x03 \x02(\x0b\x32!.livekit.proto.SentRtpStreamStats\x12\x44\n\x0fremote_outbound\x18\x04 \x02(\x0b\x32+.livekit.proto.RemoteOutboundRtpStreamStats\x1a\xc8\x01\n\x0bMediaSource\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12/\n\x06source\x18\x02 \x02(\x0b\x32\x1f.livekit.proto.MediaSourceStats\x12.\n\x05\x61udio\x18\x03 \x02(\x0b\x32\x1f.livekit.proto.AudioSourceStats\x12.\n\x05video\x18\x04 \x02(\x0b\x32\x1f.livekit.proto.VideoSourceStats\x1aq\n\x0cMediaPlayout\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12\x37\n\raudio_playout\x18\x02 \x02(\x0b\x32 .livekit.proto.AudioPlayoutStats\x1aj\n\x0ePeerConnection\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12.\n\x02pc\x18\x02 \x02(\x0b\x32\".livekit.proto.PeerConnectionStats\x1a\x64\n\x0b\x44\x61taChannel\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12+\n\x02\x64\x63\x18\x02 \x02(\x0b\x32\x1f.livekit.proto.DataChannelStats\x1ag\n\tTransport\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12\x30\n\ttransport\x18\x02 \x02(\x0b\x32\x1d.livekit.proto.TransportStats\x1at\n\rCandidatePair\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12\x39\n\x0e\x63\x61ndidate_pair\x18\x02 \x02(\x0b\x32!.livekit.proto.CandidatePairStats\x1ao\n\x0eLocalCandidate\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12\x33\n\tcandidate\x18\x02 \x02(\x0b\x32 .livekit.proto.IceCandidateStats\x1ap\n\x0fRemoteCandidate\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12\x33\n\tcandidate\x18\x02 \x02(\x0b\x32 .livekit.proto.IceCandidateStats\x1am\n\x0b\x43\x65rtificate\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12\x34\n\x0b\x63\x65rtificate\x18\x02 \x02(\x0b\x32\x1f.livekit.proto.CertificateStats\x1a^\n\x06Stream\x12(\n\x03rtc\x18\x01 \x02(\x0b\x32\x1b.livekit.proto.RtcStatsData\x12*\n\x06stream\x18\x02 \x02(\x0b\x32\x1a.livekit.proto.StreamStats\x1a\x07\n\x05TrackB\x07\n\x05stats\"-\n\x0cRtcStatsData\x12\n\n\x02id\x18\x01 \x02(\t\x12\x11\n\ttimestamp\x18\x02 \x02(\x03\"\x88\x01\n\nCodecStats\x12\x14\n\x0cpayload_type\x18\x01 \x02(\r\x12\x14\n\x0ctransport_id\x18\x02 \x02(\t\x12\x11\n\tmime_type\x18\x03 \x02(\t\x12\x12\n\nclock_rate\x18\x04 \x02(\r\x12\x10\n\x08\x63hannels\x18\x05 \x02(\r\x12\x15\n\rsdp_fmtp_line\x18\x06 \x02(\t\"T\n\x0eRtpStreamStats\x12\x0c\n\x04ssrc\x18\x01 \x02(\r\x12\x0c\n\x04kind\x18\x02 \x02(\t\x12\x14\n\x0ctransport_id\x18\x03 \x02(\t\x12\x10\n\x08\x63odec_id\x18\x04 \x02(\t\"X\n\x16ReceivedRtpStreamStats\x12\x18\n\x10packets_received\x18\x01 \x02(\x04\x12\x14\n\x0cpackets_lost\x18\x02 \x02(\x03\x12\x0e\n\x06jitter\x18\x03 \x02(\x01\"\x82\x0c\n\x15InboundRtpStreamStats\x12\x18\n\x10track_identifier\x18\x01 \x02(\t\x12\x0b\n\x03mid\x18\x02 \x02(\t\x12\x11\n\tremote_id\x18\x03 \x02(\t\x12\x16\n\x0e\x66rames_decoded\x18\x04 \x02(\r\x12\x1a\n\x12key_frames_decoded\x18\x05 \x02(\r\x12\x17\n\x0f\x66rames_rendered\x18\x06 \x02(\r\x12\x16\n\x0e\x66rames_dropped\x18\x07 \x02(\r\x12\x13\n\x0b\x66rame_width\x18\x08 \x02(\r\x12\x14\n\x0c\x66rame_height\x18\t \x02(\r\x12\x19\n\x11\x66rames_per_second\x18\n \x02(\x01\x12\x0e\n\x06qp_sum\x18\x0b \x02(\x04\x12\x19\n\x11total_decode_time\x18\x0c \x02(\x01\x12\x1f\n\x17total_inter_frame_delay\x18\r \x02(\x01\x12\'\n\x1ftotal_squared_inter_frame_delay\x18\x0e \x02(\x01\x12\x13\n\x0bpause_count\x18\x0f \x02(\r\x12\x1c\n\x14total_pause_duration\x18\x10 \x02(\x01\x12\x14\n\x0c\x66reeze_count\x18\x11 \x02(\r\x12\x1d\n\x15total_freeze_duration\x18\x12 \x02(\x01\x12&\n\x1elast_packet_received_timestamp\x18\x13 \x02(\x01\x12\x1d\n\x15header_bytes_received\x18\x14 \x02(\x04\x12\x19\n\x11packets_discarded\x18\x15 \x02(\x04\x12\x1a\n\x12\x66\x65\x63_bytes_received\x18\x16 \x02(\x04\x12\x1c\n\x14\x66\x65\x63_packets_received\x18\x17 \x02(\x04\x12\x1d\n\x15\x66\x65\x63_packets_discarded\x18\x18 \x02(\x04\x12\x16\n\x0e\x62ytes_received\x18\x19 \x02(\x04\x12\x12\n\nnack_count\x18\x1a \x02(\r\x12\x11\n\tfir_count\x18\x1b \x02(\r\x12\x11\n\tpli_count\x18\x1c \x02(\r\x12\x1e\n\x16total_processing_delay\x18\x1d \x02(\x01\x12#\n\x1b\x65stimated_playout_timestamp\x18\x1e \x02(\x01\x12\x1b\n\x13jitter_buffer_delay\x18\x1f \x02(\x01\x12\"\n\x1ajitter_buffer_target_delay\x18  \x02(\x01\x12#\n\x1bjitter_buffer_emitted_count\x18! \x02(\x04\x12#\n\x1bjitter_buffer_minimum_delay\x18\" \x02(\x01\x12\x1e\n\x16total_samples_received\x18# \x02(\x04\x12\x19\n\x11\x63oncealed_samples\x18$ \x02(\x04\x12 \n\x18silent_concealed_samples\x18% \x02(\x04\x12\x1a\n\x12\x63oncealment_events\x18& \x02(\x04\x12)\n!inserted_samples_for_deceleration\x18\' \x02(\x04\x12(\n removed_samples_for_acceleration\x18( \x02(\x04\x12\x13\n\x0b\x61udio_level\x18) \x02(\x01\x12\x1a\n\x12total_audio_energy\x18* \x02(\x01\x12\x1e\n\x16total_samples_duration\x18+ \x02(\x01\x12\x17\n\x0f\x66rames_received\x18, \x02(\x04\x12\x1e\n\x16\x64\x65\x63oder_implementation\x18- \x02(\t\x12\x12\n\nplayout_id\x18. \x02(\t\x12\x1f\n\x17power_efficient_decoder\x18/ \x02(\x08\x12.\n&frames_assembled_from_multiple_packets\x18\x30 \x02(\x04\x12\x1b\n\x13total_assembly_time\x18\x31 \x02(\x01\x12&\n\x1eretransmitted_packets_received\x18\x32 \x02(\x04\x12$\n\x1cretransmitted_bytes_received\x18\x33 \x02(\x04\x12\x10\n\x08rtx_ssrc\x18\x34 \x02(\r\x12\x10\n\x08\x66\x65\x63_ssrc\x18\x35 \x02(\r\">\n\x12SentRtpStreamStats\x12\x14\n\x0cpackets_sent\x18\x01 \x02(\x04\x12\x12\n\nbytes_sent\x18\x02 \x02(\x04\"\xd1\x07\n\x16OutboundRtpStreamStats\x12\x0b\n\x03mid\x18\x01 \x02(\t\x12\x17\n\x0fmedia_source_id\x18\x02 \x02(\t\x12\x11\n\tremote_id\x18\x03 \x02(\t\x12\x0b\n\x03rid\x18\x04 \x02(\t\x12\x19\n\x11header_bytes_sent\x18\x05 \x02(\x04\x12\"\n\x1aretransmitted_packets_sent\x18\x06 \x02(\x04\x12 \n\x18retransmitted_bytes_sent\x18\x07 \x02(\x04\x12\x10\n\x08rtx_ssrc\x18\x08 \x02(\r\x12\x16\n\x0etarget_bitrate\x18\t \x02(\x01\x12\"\n\x1atotal_encoded_bytes_target\x18\n \x02(\x04\x12\x13\n\x0b\x66rame_width\x18\x0b \x02(\r\x12\x14\n\x0c\x66rame_height\x18\x0c \x02(\r\x12\x19\n\x11\x66rames_per_second\x18\r \x02(\x01\x12\x13\n\x0b\x66rames_sent\x18\x0e \x02(\r\x12\x18\n\x10huge_frames_sent\x18\x0f \x02(\r\x12\x16\n\x0e\x66rames_encoded\x18\x10 \x02(\r\x12\x1a\n\x12key_frames_encoded\x18\x11 \x02(\r\x12\x0e\n\x06qp_sum\x18\x12 \x02(\x04\x12\x19\n\x11total_encode_time\x18\x13 \x02(\x01\x12\x1f\n\x17total_packet_send_delay\x18\x14 \x02(\x01\x12I\n\x19quality_limitation_reason\x18\x15 \x02(\x0e\x32&.livekit.proto.QualityLimitationReason\x12k\n\x1cquality_limitation_durations\x18\x16 \x03(\x0b\x32\x45.livekit.proto.OutboundRtpStreamStats.QualityLimitationDurationsEntry\x12-\n%quality_limitation_resolution_changes\x18\x17 \x02(\r\x12\x12\n\nnack_count\x18\x18 \x02(\r\x12\x11\n\tfir_count\x18\x19 \x02(\r\x12\x11\n\tpli_count\x18\x1a \x02(\r\x12\x1e\n\x16\x65ncoder_implementation\x18\x1b \x02(\t\x12\x1f\n\x17power_efficient_encoder\x18\x1c \x02(\x08\x12\x0e\n\x06\x61\x63tive\x18\x1d \x02(\x08\x12\x18\n\x10scalability_mode\x18\x1e \x02(\t\x1a\x41\n\x1fQualityLimitationDurationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\"\xa4\x01\n\x1bRemoteInboundRtpStreamStats\x12\x10\n\x08local_id\x18\x01 \x02(\t\x12\x17\n\x0fround_trip_time\x18\x02 \x02(\x01\x12\x1d\n\x15total_round_trip_time\x18\x03 \x02(\x01\x12\x15\n\rfraction_lost\x18\x04 \x02(\x01\x12$\n\x1cround_trip_time_measurements\x18\x05 \x02(\x04\"\xbe\x01\n\x1cRemoteOutboundRtpStreamStats\x12\x10\n\x08local_id\x18\x01 \x02(\t\x12\x18\n\x10remote_timestamp\x18\x02 \x02(\x01\x12\x14\n\x0creports_sent\x18\x03 \x02(\x04\x12\x17\n\x0fround_trip_time\x18\x04 \x02(\x01\x12\x1d\n\x15total_round_trip_time\x18\x05 \x02(\x01\x12$\n\x1cround_trip_time_measurements\x18\x06 \x02(\x04\":\n\x10MediaSourceStats\x12\x18\n\x10track_identifier\x18\x01 \x02(\t\x12\x0c\n\x04kind\x18\x02 \x02(\t\"\xa2\x02\n\x10\x41udioSourceStats\x12\x13\n\x0b\x61udio_level\x18\x01 \x02(\x01\x12\x1a\n\x12total_audio_energy\x18\x02 \x02(\x01\x12\x1e\n\x16total_samples_duration\x18\x03 \x02(\x01\x12\x18\n\x10\x65\x63ho_return_loss\x18\x04 \x02(\x01\x12$\n\x1c\x65\x63ho_return_loss_enhancement\x18\x05 \x02(\x01\x12 \n\x18\x64ropped_samples_duration\x18\x06 \x02(\x01\x12\x1e\n\x16\x64ropped_samples_events\x18\x07 \x02(\r\x12\x1b\n\x13total_capture_delay\x18\x08 \x02(\x01\x12\x1e\n\x16total_samples_captured\x18\t \x02(\x04\"\\\n\x10VideoSourceStats\x12\r\n\x05width\x18\x01 \x02(\r\x12\x0e\n\x06height\x18\x02 \x02(\r\x12\x0e\n\x06\x66rames\x18\x03 \x02(\r\x12\x19\n\x11\x66rames_per_second\x18\x04 \x02(\x01\"\xc5\x01\n\x11\x41udioPlayoutStats\x12\x0c\n\x04kind\x18\x01 \x02(\t\x12$\n\x1csynthesized_samples_duration\x18\x02 \x02(\x01\x12\"\n\x1asynthesized_samples_events\x18\x03 \x02(\r\x12\x1e\n\x16total_samples_duration\x18\x04 \x02(\x01\x12\x1b\n\x13total_playout_delay\x18\x05 \x02(\x01\x12\x1b\n\x13total_samples_count\x18\x06 \x02(\x04\"Q\n\x13PeerConnectionStats\x12\x1c\n\x14\x64\x61ta_channels_opened\x18\x01 \x02(\r\x12\x1c\n\x14\x64\x61ta_channels_closed\x18\x02 \x02(\r\"\xe2\x01\n\x10\x44\x61taChannelStats\x12\r\n\x05label\x18\x01 \x02(\t\x12\x10\n\x08protocol\x18\x02 \x02(\t\x12\x1f\n\x17\x64\x61ta_channel_identifier\x18\x03 \x02(\x05\x12.\n\x05state\x18\x04 \x01(\x0e\x32\x1f.livekit.proto.DataChannelState\x12\x15\n\rmessages_sent\x18\x05 \x02(\r\x12\x12\n\nbytes_sent\x18\x06 \x02(\x04\x12\x19\n\x11messages_received\x18\x07 \x02(\r\x12\x16\n\x0e\x62ytes_received\x18\x08 \x02(\x04\"\x9c\x04\n\x0eTransportStats\x12\x14\n\x0cpackets_sent\x18\x01 \x02(\x04\x12\x18\n\x10packets_received\x18\x02 \x02(\x04\x12\x12\n\nbytes_sent\x18\x03 \x02(\x04\x12\x16\n\x0e\x62ytes_received\x18\x04 \x02(\x04\x12(\n\x08ice_role\x18\x05 \x02(\x0e\x32\x16.livekit.proto.IceRole\x12#\n\x1bice_local_username_fragment\x18\x06 \x02(\t\x12\x35\n\ndtls_state\x18\x07 \x01(\x0e\x32!.livekit.proto.DtlsTransportState\x12\x33\n\tice_state\x18\x08 \x01(\x0e\x32 .livekit.proto.IceTransportState\x12\"\n\x1aselected_candidate_pair_id\x18\t \x02(\t\x12\x1c\n\x14local_certificate_id\x18\n \x02(\t\x12\x1d\n\x15remote_certificate_id\x18\x0b \x02(\t\x12\x13\n\x0btls_version\x18\x0c \x02(\t\x12\x13\n\x0b\x64tls_cipher\x18\r \x02(\t\x12*\n\tdtls_role\x18\x0e \x02(\x0e\x32\x17.livekit.proto.DtlsRole\x12\x13\n\x0bsrtp_cipher\x18\x0f \x02(\t\x12\'\n\x1fselected_candidate_pair_changes\x18\x10 \x02(\r\"\xa4\x05\n\x12\x43\x61ndidatePairStats\x12\x14\n\x0ctransport_id\x18\x01 \x02(\t\x12\x1a\n\x12local_candidate_id\x18\x02 \x02(\t\x12\x1b\n\x13remote_candidate_id\x18\x03 \x02(\t\x12\x33\n\x05state\x18\x04 \x01(\x0e\x32$.livekit.proto.IceCandidatePairState\x12\x11\n\tnominated\x18\x05 \x02(\x08\x12\x14\n\x0cpackets_sent\x18\x06 \x02(\x04\x12\x18\n\x10packets_received\x18\x07 \x02(\x04\x12\x12\n\nbytes_sent\x18\x08 \x02(\x04\x12\x16\n\x0e\x62ytes_received\x18\t \x02(\x04\x12\"\n\x1alast_packet_sent_timestamp\x18\n \x02(\x01\x12&\n\x1elast_packet_received_timestamp\x18\x0b \x02(\x01\x12\x1d\n\x15total_round_trip_time\x18\x0c \x02(\x01\x12\x1f\n\x17\x63urrent_round_trip_time\x18\r \x02(\x01\x12\"\n\x1a\x61vailable_outgoing_bitrate\x18\x0e \x02(\x01\x12\"\n\x1a\x61vailable_incoming_bitrate\x18\x0f \x02(\x01\x12\x19\n\x11requests_received\x18\x10 \x02(\x04\x12\x15\n\rrequests_sent\x18\x11 \x02(\x04\x12\x1a\n\x12responses_received\x18\x12 \x02(\x04\x12\x16\n\x0eresponses_sent\x18\x13 \x02(\x04\x12\x1d\n\x15\x63onsent_requests_sent\x18\x14 \x02(\x04\x12!\n\x19packets_discarded_on_send\x18\x15 \x02(\r\x12\x1f\n\x17\x62ytes_discarded_on_send\x18\x16 \x02(\x04\"\x89\x03\n\x11IceCandidateStats\x12\x14\n\x0ctransport_id\x18\x01 \x02(\t\x12\x0f\n\x07\x61\x64\x64ress\x18\x02 \x02(\t\x12\x0c\n\x04port\x18\x03 \x02(\x05\x12\x10\n\x08protocol\x18\x04 \x02(\t\x12\x37\n\x0e\x63\x61ndidate_type\x18\x05 \x01(\x0e\x32\x1f.livekit.proto.IceCandidateType\x12\x10\n\x08priority\x18\x06 \x02(\x05\x12\x0b\n\x03url\x18\x07 \x02(\t\x12\x41\n\x0erelay_protocol\x18\x08 \x01(\x0e\x32).livekit.proto.IceServerTransportProtocol\x12\x12\n\nfoundation\x18\t \x02(\t\x12\x17\n\x0frelated_address\x18\n \x02(\t\x12\x14\n\x0crelated_port\x18\x0b \x02(\x05\x12\x19\n\x11username_fragment\x18\x0c \x02(\t\x12\x34\n\x08tcp_type\x18\r \x01(\x0e\x32\".livekit.proto.IceTcpCandidateType\"\x81\x01\n\x10\x43\x65rtificateStats\x12\x13\n\x0b\x66ingerprint\x18\x01 \x02(\t\x12\x1d\n\x15\x66ingerprint_algorithm\x18\x02 \x02(\t\x12\x1a\n\x12\x62\x61se64_certificate\x18\x03 \x02(\t\x12\x1d\n\x15issuer_certificate_id\x18\x04 \x02(\t\"4\n\x0bStreamStats\x12\n\n\x02id\x18\x01 \x02(\t\x12\x19\n\x11stream_identifier\x18\x02 \x02(\t*Q\n\x10\x44\x61taChannelState\x12\x11\n\rDC_CONNECTING\x10\x00\x12\x0b\n\x07\x44\x43_OPEN\x10\x01\x12\x0e\n\nDC_CLOSING\x10\x02\x12\r\n\tDC_CLOSED\x10\x03*r\n\x17QualityLimitationReason\x12\x13\n\x0fLIMITATION_NONE\x10\x00\x12\x12\n\x0eLIMITATION_CPU\x10\x01\x12\x18\n\x14LIMITATION_BANDWIDTH\x10\x02\x12\x14\n\x10LIMITATION_OTHER\x10\x03*C\n\x07IceRole\x12\x0f\n\x0bICE_UNKNOWN\x10\x00\x12\x13\n\x0fICE_CONTROLLING\x10\x01\x12\x12\n\x0eICE_CONTROLLED\x10\x02*\x9f\x01\n\x12\x44tlsTransportState\x12\x16\n\x12\x44TLS_TRANSPORT_NEW\x10\x00\x12\x1d\n\x19\x44TLS_TRANSPORT_CONNECTING\x10\x01\x12\x1c\n\x18\x44TLS_TRANSPORT_CONNECTED\x10\x02\x12\x19\n\x15\x44TLS_TRANSPORT_CLOSED\x10\x03\x12\x19\n\x15\x44TLS_TRANSPORT_FAILED\x10\x04*\xd4\x01\n\x11IceTransportState\x12\x15\n\x11ICE_TRANSPORT_NEW\x10\x00\x12\x1a\n\x16ICE_TRANSPORT_CHECKING\x10\x01\x12\x1b\n\x17ICE_TRANSPORT_CONNECTED\x10\x02\x12\x1b\n\x17ICE_TRANSPORT_COMPLETED\x10\x03\x12\x1e\n\x1aICE_TRANSPORT_DISCONNECTED\x10\x04\x12\x18\n\x14ICE_TRANSPORT_FAILED\x10\x05\x12\x18\n\x14ICE_TRANSPORT_CLOSED\x10\x06*>\n\x08\x44tlsRole\x12\x0f\n\x0b\x44TLS_CLIENT\x10\x00\x12\x0f\n\x0b\x44TLS_SERVER\x10\x01\x12\x10\n\x0c\x44TLS_UNKNOWN\x10\x02*u\n\x15IceCandidatePairState\x12\x0f\n\x0bPAIR_FROZEN\x10\x00\x12\x10\n\x0cPAIR_WAITING\x10\x01\x12\x14\n\x10PAIR_IN_PROGRESS\x10\x02\x12\x0f\n\x0bPAIR_FAILED\x10\x03\x12\x12\n\x0ePAIR_SUCCEEDED\x10\x04*=\n\x10IceCandidateType\x12\x08\n\x04HOST\x10\x00\x12\t\n\x05SRFLX\x10\x01\x12\t\n\x05PRFLX\x10\x02\x12\t\n\x05RELAY\x10\x03*U\n\x1aIceServerTransportProtocol\x12\x11\n\rTRANSPORT_UDP\x10\x00\x12\x11\n\rTRANSPORT_TCP\x10\x01\x12\x11\n\rTRANSPORT_TLS\x10\x02*T\n\x13IceTcpCandidateType\x12\x14\n\x10\x43\x41NDIDATE_ACTIVE\x10\x00\x12\x15\n\x11\x43\x41NDIDATE_PASSIVE\x10\x01\x12\x10\n\x0c\x43\x41NDIDATE_SO\x10\x02\x42\x10\xaa\x02\rLiveKit.Proto')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'stats_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\rLiveKit.Proto'
  _globals['_OUTBOUNDRTPSTREAMSTATS_QUALITYLIMITATIONDURATIONSENTRY']._options = None
  _globals['_OUTBOUNDRTPSTREAMSTATS_QUALITYLIMITATIONDURATIONSENTRY']._serialized_options = b'8\001'
  _globals['_DATACHANNELSTATE']._serialized_start=9282
  _globals['_DATACHANNELSTATE']._serialized_end=9363
  _globals['_QUALITYLIMITATIONREASON']._serialized_start=9365
  _globals['_QUALITYLIMITATIONREASON']._serialized_end=9479
  _globals['_ICEROLE']._serialized_start=9481
  _globals['_ICEROLE']._serialized_end=9548
  _globals['_DTLSTRANSPORTSTATE']._serialized_start=9551
  _globals['_DTLSTRANSPORTSTATE']._serialized_end=9710
  _globals['_ICETRANSPORTSTATE']._serialized_start=9713
  _globals['_ICETRANSPORTSTATE']._serialized_end=9925
  _globals['_DTLSROLE']._serialized_start=9927
  _globals['_DTLSROLE']._serialized_end=9989
  _globals['_ICECANDIDATEPAIRSTATE']._serialized_start=9991
  _globals['_ICECANDIDATEPAIRSTATE']._serialized_end=10108
  _globals['_ICECANDIDATETYPE']._serialized_start=10110
  _globals['_ICECANDIDATETYPE']._serialized_end=10171
  _globals['_ICESERVERTRANSPORTPROTOCOL']._serialized_start=10173
  _globals['_ICESERVERTRANSPORTPROTOCOL']._serialized_end=10258
  _globals['_ICETCPCANDIDATETYPE']._serialized_start=10260
  _globals['_ICETCPCANDIDATETYPE']._serialized_end=10344
  _globals['_RTCSTATS']._serialized_start=31
  _globals['_RTCSTATS']._serialized_end=3210
  _globals['_RTCSTATS_CODEC']._serialized_start=1024
  _globals['_RTCSTATS_CODEC']._serialized_end=1115
  _globals['_RTCSTATS_INBOUNDRTP']._serialized_start=1118
  _globals['_RTCSTATS_INBOUNDRTP']._serialized_end=1331
  _globals['_RTCSTATS_OUTBOUNDRTP']._serialized_start=1334
  _globals['_RTCSTATS_OUTBOUNDRTP']._serialized_end=1542
  _globals['_RTCSTATS_REMOTEINBOUNDRTP']._serialized_start=1545
  _globals['_RTCSTATS_REMOTEINBOUNDRTP']._serialized_end=1777
  _globals['_RTCSTATS_REMOTEOUTBOUNDRTP']._serialized_start=1780
  _globals['_RTCSTATS_REMOTEOUTBOUNDRTP']._serialized_end=2007
  _globals['_RTCSTATS_MEDIASOURCE']._serialized_start=2010
  _globals['_RTCSTATS_MEDIASOURCE']._serialized_end=2210
  _globals['_RTCSTATS_MEDIAPLAYOUT']._serialized_start=2212
  _globals['_RTCSTATS_MEDIAPLAYOUT']._serialized_end=2325
  _globals['_RTCSTATS_PEERCONNECTION']._serialized_start=2327
  _globals['_RTCSTATS_PEERCONNECTION']._serialized_end=2433
  _globals['_RTCSTATS_DATACHANNEL']._serialized_start=2435
  _globals['_RTCSTATS_DATACHANNEL']._serialized_end=2535
  _globals['_RTCSTATS_TRANSPORT']._serialized_start=2537
  _globals['_RTCSTATS_TRANSPORT']._serialized_end=2640
  _globals['_RTCSTATS_CANDIDATEPAIR']._serialized_start=2642
  _globals['_RTCSTATS_CANDIDATEPAIR']._serialized_end=2758
  _globals['_RTCSTATS_LOCALCANDIDATE']._serialized_start=2760
  _globals['_RTCSTATS_LOCALCANDIDATE']._serialized_end=2871
  _globals['_RTCSTATS_REMOTECANDIDATE']._serialized_start=2873
  _globals['_RTCSTATS_REMOTECANDIDATE']._serialized_end=2985
  _globals['_RTCSTATS_CERTIFICATE']._serialized_start=2987
  _globals['_RTCSTATS_CERTIFICATE']._serialized_end=3096
  _globals['_RTCSTATS_STREAM']._serialized_start=3098
  _globals['_RTCSTATS_STREAM']._serialized_end=3192
  _globals['_RTCSTATS_TRACK']._serialized_start=3194
  _globals['_RTCSTATS_TRACK']._serialized_end=3201
  _globals['_RTCSTATSDATA']._serialized_start=3212
  _globals['_RTCSTATSDATA']._serialized_end=3257
  _globals['_CODECSTATS']._serialized_start=3260
  _globals['_CODECSTATS']._serialized_end=3396
  _globals['_RTPSTREAMSTATS']._serialized_start=3398
  _globals['_RTPSTREAMSTATS']._serialized_end=3482
  _globals['_RECEIVEDRTPSTREAMSTATS']._serialized_start=3484
  _globals['_RECEIVEDRTPSTREAMSTATS']._serialized_end=3572
  _globals['_INBOUNDRTPSTREAMSTATS']._serialized_start=3575
  _globals['_INBOUNDRTPSTREAMSTATS']._serialized_end=5113
  _globals['_SENTRTPSTREAMSTATS']._serialized_start=5115
  _globals['_SENTRTPSTREAMSTATS']._serialized_end=5177
  _globals['_OUTBOUNDRTPSTREAMSTATS']._serialized_start=5180
  _globals['_OUTBOUNDRTPSTREAMSTATS']._serialized_end=6157
  _globals['_OUTBOUNDRTPSTREAMSTATS_QUALITYLIMITATIONDURATIONSENTRY']._serialized_start=6092
  _globals['_OUTBOUNDRTPSTREAMSTATS_QUALITYLIMITATIONDURATIONSENTRY']._serialized_end=6157
  _globals['_REMOTEINBOUNDRTPSTREAMSTATS']._serialized_start=6160
  _globals['_REMOTEINBOUNDRTPSTREAMSTATS']._serialized_end=6324
  _globals['_REMOTEOUTBOUNDRTPSTREAMSTATS']._serialized_start=6327
  _globals['_REMOTEOUTBOUNDRTPSTREAMSTATS']._serialized_end=6517
  _globals['_MEDIASOURCESTATS']._serialized_start=6519
  _globals['_MEDIASOURCESTATS']._serialized_end=6577
  _globals['_AUDIOSOURCESTATS']._serialized_start=6580
  _globals['_AUDIOSOURCESTATS']._serialized_end=6870
  _globals['_VIDEOSOURCESTATS']._serialized_start=6872
  _globals['_VIDEOSOURCESTATS']._serialized_end=6964
  _globals['_AUDIOPLAYOUTSTATS']._serialized_start=6967
  _globals['_AUDIOPLAYOUTSTATS']._serialized_end=7164
  _globals['_PEERCONNECTIONSTATS']._serialized_start=7166
  _globals['_PEERCONNECTIONSTATS']._serialized_end=7247
  _globals['_DATACHANNELSTATS']._serialized_start=7250
  _globals['_DATACHANNELSTATS']._serialized_end=7476
  _globals['_TRANSPORTSTATS']._serialized_start=7479
  _globals['_TRANSPORTSTATS']._serialized_end=8019
  _globals['_CANDIDATEPAIRSTATS']._serialized_start=8022
  _globals['_CANDIDATEPAIRSTATS']._serialized_end=8698
  _globals['_ICECANDIDATESTATS']._serialized_start=8701
  _globals['_ICECANDIDATESTATS']._serialized_end=9094
  _globals['_CERTIFICATESTATS']._serialized_start=9097
  _globals['_CERTIFICATESTATS']._serialized_end=9226
  _globals['_STREAMSTATS']._serialized_start=9228
  _globals['_STREAMSTATS']._serialized_end=9280
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2025 LiveKit, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
from . import handle_pb2
import sys
from . import track_pb2
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _SoxResamplerDataType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _SoxResamplerDataTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_SoxResamplerDataType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SOXR_DATATYPE_INT16I: _SoxResamplerDataType.ValueType  # 0
    """TODO(theomonnom): support other datatypes (shouldn't really be needed)"""
    SOXR_DATATYPE_INT16S: _SoxResamplerDataType.ValueType  # 1

class SoxResamplerDataType(_SoxResamplerDataType, metaclass=_SoxResamplerDataTypeEnumTypeWrapper): ...

SOXR_DATATYPE_INT16I: SoxResamplerDataType.ValueType  # 0
"""TODO(theomonnom): support other datatypes (shouldn't really be needed)"""
SOXR_DATATYPE_INT16S: SoxResamplerDataType.ValueType  # 1
global___SoxResamplerDataType = SoxResamplerDataType

class _SoxQualityRecipe:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _SoxQualityRecipeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_SoxQualityRecipe.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SOXR_QUALITY_QUICK: _SoxQualityRecipe.ValueType  # 0
    SOXR_QUALITY_LOW: _SoxQualityRecipe.ValueType  # 1
    SOXR_QUALITY_MEDIUM: _SoxQualityRecipe.ValueType  # 2
    SOXR_QUALITY_HIGH: _SoxQualityRecipe.ValueType  # 3
    SOXR_QUALITY_VERYHIGH: _SoxQualityRecipe.ValueType  # 4

class SoxQualityRecipe(_SoxQualityRecipe, metaclass=_SoxQualityRecipeEnumTypeWrapper): ...

SOXR_QUALITY_QUICK: SoxQualityRecipe.ValueType  # 0
SOXR_QUALITY_LOW: SoxQualityRecipe.ValueType  # 1
SOXR_QUALITY_MEDIUM: SoxQualityRecipe.ValueType  # 2
SOXR_QUALITY_HIGH: SoxQualityRecipe.ValueType  # 3
SOXR_QUALITY_VERYHIGH: SoxQualityRecipe.ValueType  # 4
global___SoxQualityRecipe = SoxQualityRecipe

class _SoxFlagBits:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _SoxFlagBitsEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_SoxFlagBits.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SOXR_ROLLOFF_SMALL: _SoxFlagBits.ValueType  # 0
    """1 << 0"""
    SOXR_ROLLOFF_MEDIUM: _SoxFlagBits.ValueType  # 1
    """1 << 1"""
    SOXR_ROLLOFF_NONE: _SoxFlagBits.ValueType  # 2
    """1 << 2"""
    SOXR_HIGH_PREC_CLOCK: _SoxFlagBits.ValueType  # 3
    """1 << 3"""
    SOXR_DOUBLE_PRECISION: _SoxFlagBits.ValueType  # 4
    """1 << 4"""
    SOXR_VR: _SoxFlagBits.ValueType  # 5
    """1 << 5"""

class SoxFlagBits(_SoxFlagBits, metaclass=_SoxFlagBitsEnumTypeWrapper): ...

SOXR_ROLLOFF_SMALL: SoxFlagBits.ValueType  # 0
"""1 << 0"""
SOXR_ROLLOFF_MEDIUM: SoxFlagBits.ValueType  # 1
"""1 << 1"""
SOXR_ROLLOFF_NONE: SoxFlagBits.ValueType  # 2
"""1 << 2"""
SOXR_HIGH_PREC_CLOCK: SoxFlagBits.ValueType  # 3
"""1 << 3"""
SOXR_DOUBLE_PRECISION: SoxFlagBits.ValueType  # 4
"""1 << 4"""
SOXR_VR: SoxFlagBits.ValueType  # 5
"""1 << 5"""
global___SoxFlagBits = SoxFlagBits

class _AudioStreamType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _AudioStreamTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_AudioStreamType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    AUDIO_STREAM_NATIVE: _AudioStreamType.ValueType  # 0
    AUDIO_STREAM_HTML: _AudioStreamType.ValueType  # 1

class AudioStreamType(_AudioStreamType, metaclass=_AudioStreamTypeEnumTypeWrapper):
    """
    AudioStream
    """

AUDIO_STREAM_NATIVE: AudioStreamType.ValueType  # 0
AUDIO_STREAM_HTML: AudioStreamType.ValueType  # 1
global___AudioStreamType = AudioStreamType

class _AudioSourceType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _AudioSourceTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_AudioSourceType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    AUDIO_SOURCE_NATIVE: _AudioSourceType.ValueType  # 0

class AudioSourceType(_AudioSourceType, metaclass=_AudioSourceTypeEnumTypeWrapper): ...

AUDIO_SOURCE_NATIVE: AudioSourceType.ValueType  # 0
global___AudioSourceType = AudioSourceType

@typing.final
class NewAudioStreamRequest(google.protobuf.message.Message):
    """Create a new AudioStream
    AudioStream is used to receive audio frames from a track
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TRACK_HANDLE_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    SAMPLE_RATE_FIELD_NUMBER: builtins.int
    NUM_CHANNELS_FIELD_NUMBER: builtins.int
    AUDIO_FILTER_MODULE_ID_FIELD_NUMBER: builtins.int
    AUDIO_FILTER_OPTIONS_FIELD_NUMBER: builtins.int
    FRAME_SIZE_MS_FIELD_NUMBER: builtins.int
    track_handle: builtins.int
    type: global___AudioStreamType.ValueType
    sample_rate: builtins.int
    num_channels: builtins.int
    audio_filter_module_id: builtins.str
    """Unique identifier passed in LoadAudioFilterPluginRequest"""
    audio_filter_options: builtins.str
    frame_size_ms: builtins.int
    def __init__(
        self,
        *,
        track_handle: builtins.int | None = ...,
        type: global___AudioStreamType.ValueType | None = ...,
        sample_rate: builtins.int | None = ...,
        num_channels: builtins.int | None = ...,
        audio_filter_module_id: builtins.str | None = ...,
        audio_filter_options: builtins.str | None = ...,
        frame_size_ms: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["audio_filter_module_id", b"audio_filter_module_id", "audio_filter_options", b"audio_filter_options", "frame_size_ms", b"frame_size_ms", "num_channels", b"num_channels", "sample_rate", b"sample_rate", "track_handle", b"track_handle", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["audio_filter_module_id", b"audio_filter_module_id", "audio_filter_options", b"audio_filter_options", "frame_size_ms", b"frame_size_ms", "num_channels", b"num_channels", "sample_rate", b"sample_rate", "track_handle", b"track_handle", "type", b"type"]) -> None: ...

global___NewAudioStreamRequest = NewAudioStreamRequest

@typing.final
class NewAudioStreamResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STREAM_FIELD_NUMBER: builtins.int
    @property
    def stream(self) -> global___OwnedAudioStream: ...
    def __init__(
        self,
        *,
        stream: global___OwnedAudioStream | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["stream", b"stream"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["stream", b"stream"]) -> None: ...

global___NewAudioStreamResponse = NewAudioStreamResponse

@typing.final
class AudioStreamFromParticipantRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PARTICIPANT_HANDLE_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    TRACK_SOURCE_FIELD_NUMBER: builtins.int
    SAMPLE_RATE_FIELD_NUMBER: builtins.int
    NUM_CHANNELS_FIELD_NUMBER: builtins.int
    AUDIO_FILTER_MODULE_ID_FIELD_NUMBER: builtins.int
    AUDIO_FILTER_OPTIONS_FIELD_NUMBER: builtins.int
    FRAME_SIZE_MS_FIELD_NUMBER: builtins.int
    participant_handle: builtins.int
    type: global___AudioStreamType.ValueType
    track_source: track_pb2.TrackSource.ValueType
    sample_rate: builtins.int
    num_channels: builtins.int
    audio_filter_module_id: builtins.str
    audio_filter_options: builtins.str
    frame_size_ms: builtins.int
    def __init__(
        self,
        *,
        participant_handle: builtins.int | None = ...,
        type: global___AudioStreamType.ValueType | None = ...,
        track_source: track_pb2.TrackSource.ValueType | None = ...,
        sample_rate: builtins.int | None = ...,
        num_channels: builtins.int | None = ...,
        audio_filter_module_id: builtins.str | None = ...,
        audio_filter_options: builtins.str | None = ...,
        frame_size_ms: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["audio_filter_module_id", b"audio_filter_module_id", "audio_filter_options", b"audio_filter_options", "frame_size_ms", b"frame_size_ms", "num_channels", b"num_channels", "participant_handle", b"participant_handle", "sample_rate", b"sample_rate", "track_source", b"track_source", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["audio_filter_module_id", b"audio_filter_module_id", "audio_filter_options", b"audio_filter_options", "frame_size_ms", b"frame_size_ms", "num_channels", b"num_channels", "participant_handle", b"participant_handle", "sample_rate", b"sample_rate", "track_source", b"track_source", "type", b"type"]) -> None: ...

global___AudioStreamFromParticipantRequest = AudioStreamFromParticipantRequest

@typing.final
class AudioStreamFromParticipantResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STREAM_FIELD_NUMBER: builtins.int
    @property
    def stream(self) -> global___OwnedAudioStream: ...
    def __init__(
        self,
        *,
        stream: global___OwnedAudioStream | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["stream", b"stream"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["stream", b"stream"]) -> None: ...

global___AudioStreamFromParticipantResponse = AudioStreamFromParticipantResponse

@typing.final
class NewAudioSourceRequest(google.protobuf.message.Message):
    """Create a new AudioSource"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_FIELD_NUMBER: builtins.int
    OPTIONS_FIELD_NUMBER: builtins.int
    SAMPLE_RATE_FIELD_NUMBER: builtins.int
    NUM_CHANNELS_FIELD_NUMBER: builtins.int
    QUEUE_SIZE_MS_FIELD_NUMBER: builtins.int
    type: global___AudioSourceType.ValueType
    sample_rate: builtins.int
    num_channels: builtins.int
    queue_size_ms: builtins.int
    @property
    def options(self) -> global___AudioSourceOptions: ...
    def __init__(
        self,
        *,
        type: global___AudioSourceType.ValueType | None = ...,
        options: global___AudioSourceOptions | None = ...,
        sample_rate: builtins.int | None = ...,
        num_channels: builtins.int | None = ...,
        queue_size_ms: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["num_channels", b"num_channels", "options", b"options", "queue_size_ms", b"queue_size_ms", "sample_rate", b"sample_rate", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["num_channels", b"num_channels", "options", b"options", "queue_size_ms", b"queue_size_ms", "sample_rate", b"sample_rate", "type", b"type"]) -> None: ...

global___NewAudioSourceRequest = NewAudioSourceRequest

@typing.final
class NewAudioSourceResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOURCE_FIELD_NUMBER: builtins.int
    @property
    def source(self) -> global___OwnedAudioSource: ...
    def __init__(
        self,
        *,
        source: global___OwnedAudioSource | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["source", b"source"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["source", b"source"]) -> None: ...

global___NewAudioSourceResponse = NewAudioSourceResponse

@typing.final
class CaptureAudioFrameRequest(google.protobuf.message.Message):
    """Push a frame to an AudioSource 
    The data provided must be available as long as the client receive the callback.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOURCE_HANDLE_FIELD_NUMBER: builtins.int
    BUFFER_FIELD_NUMBER: builtins.int
    source_handle: builtins.int
    @property
    def buffer(self) -> global___AudioFrameBufferInfo: ...
    def __init__(
        self,
        *,
        source_handle: builtins.int | None = ...,
        buffer: global___AudioFrameBufferInfo | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["buffer", b"buffer", "source_handle", b"source_handle"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["buffer", b"buffer", "source_handle", b"source_handle"]) -> None: ...

global___CaptureAudioFrameRequest = CaptureAudioFrameRequest

@typing.final
class CaptureAudioFrameResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ASYNC_ID_FIELD_NUMBER: builtins.int
    async_id: builtins.int
    def __init__(
        self,
        *,
        async_id: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["async_id", b"async_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["async_id", b"async_id"]) -> None: ...

global___CaptureAudioFrameResponse = CaptureAudioFrameResponse

@typing.final
class CaptureAudioFrameCallback(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ASYNC_ID_FIELD_NUMBER: builtins.int
    ERROR_FIELD_NUMBER: builtins.int
    async_id: builtins.int
    error: builtins.str
    def __init__(
        self,
        *,
        async_id: builtins.int | None = ...,
        error: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["async_id", b"async_id", "error", b"error"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["async_id", b"async_id", "error", b"error"]) -> None: ...

global___CaptureAudioFrameCallback = CaptureAudioFrameCallback

@typing.final
class ClearAudioBufferRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOURCE_HANDLE_FIELD_NUMBER: builtins.int
    source_handle: builtins.int
    def __init__(
        self,
        *,
        source_handle: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["source_handle", b"source_handle"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["source_handle", b"source_handle"]) -> None: ...

global___ClearAudioBufferRequest = ClearAudioBufferRequest

@typing.final
class ClearAudioBufferResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ClearAudioBufferResponse = ClearAudioBufferResponse

@typing.final
class NewAudioResamplerRequest(google.protobuf.message.Message):
    """Create a new AudioResampler"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___NewAudioResamplerRequest = NewAudioResamplerRequest

@typing.final
class NewAudioResamplerResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESAMPLER_FIELD_NUMBER: builtins.int
    @property
    def resampler(self) -> global___OwnedAudioResampler: ...
    def __init__(
        self,
        *,
        resampler: global___OwnedAudioResampler | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["resampler", b"resampler"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["resampler", b"resampler"]) -> None: ...

global___NewAudioResamplerResponse = NewAudioResamplerResponse

@typing.final
class RemixAndResampleRequest(google.protobuf.message.Message):
    """Remix and resample an audio frame"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESAMPLER_HANDLE_FIELD_NUMBER: builtins.int
    BUFFER_FIELD_NUMBER: builtins.int
    NUM_CHANNELS_FIELD_NUMBER: builtins.int
    SAMPLE_RATE_FIELD_NUMBER: builtins.int
    resampler_handle: builtins.int
    num_channels: builtins.int
    sample_rate: builtins.int
    @property
    def buffer(self) -> global___AudioFrameBufferInfo: ...
    def __init__(
        self,
        *,
        resampler_handle: builtins.int | None = ...,
        buffer: global___AudioFrameBufferInfo | None = ...,
        num_channels: builtins.int | None = ...,
        sample_rate: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["buffer", b"buffer", "num_channels", b"num_channels", "resampler_handle", b"resampler_handle", "sample_rate", b"sample_rate"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["buffer", b"buffer", "num_channels", b"num_channels", "resampler_handle", b"resampler_handle", "sample_rate", b"sample_rate"]) -> None: ...

global___RemixAndResampleRequest = RemixAndResampleRequest

@typing.final
class RemixAndResampleResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUFFER_FIELD_NUMBER: builtins.int
    @property
    def buffer(self) -> global___OwnedAudioFrameBuffer: ...
    def __init__(
        self,
        *,
        buffer: global___OwnedAudioFrameBuffer | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["buffer", b"buffer"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["buffer", b"buffer"]) -> None: ...

global___RemixAndResampleResponse = RemixAndResampleResponse

@typing.final
class NewApmRequest(google.protobuf.message.Message):
    """AEC"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ECHO_CANCELLER_ENABLED_FIELD_NUMBER: builtins.int
    GAIN_CONTROLLER_ENABLED_FIELD_NUMBER: builtins.int
    HIGH_PASS_FILTER_ENABLED_FIELD_NUMBER: builtins.int
    NOISE_SUPPRESSION_ENABLED_FIELD_NUMBER: builtins.int
    echo_canceller_enabled: builtins.bool
    gain_controller_enabled: builtins.bool
    high_pass_filter_enabled: builtins.bool
    noise_suppression_enabled: builtins.bool
    def __init__(
        self,
        *,
        echo_canceller_enabled: builtins.bool | None = ...,
        gain_controller_enabled: builtins.bool | None = ...,
        high_pass_filter_enabled: builtins.bool | None = ...,
        noise_suppression_enabled: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["echo_canceller_enabled", b"echo_canceller_enabled", "gain_controller_enabled", b"gain_controller_enabled", "high_pass_filter_enabled", b"high_pass_filter_enabled", "noise_suppression_enabled", b"noise_suppression_enabled"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["echo_canceller_enabled", b"echo_canceller_enabled", "gain_controller_enabled", b"gain_controller_enabled", "high_pass_filter_enabled", b"high_pass_filter_enabled", "noise_suppression_enabled", b"noise_suppression_enabled"]) -> None: ...

global___NewApmRequest = NewApmRequest

@typing.final
class NewApmResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    APM_FIELD_NUMBER: builtins.int
    @property
    def apm(self) -> global___OwnedApm: ...
    def __init__(
        self,
        *,
        apm: global___OwnedApm | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["apm", b"apm"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["apm", b"apm"]) -> None: ...

global___NewApmResponse = NewApmResponse

@typing.final
class ApmProcessStreamRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    APM_HANDLE_FIELD_NUMBER: builtins.int
    DATA_PTR_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    SAMPLE_RATE_FIELD_NUMBER: builtins.int
    NUM_CHANNELS_FIELD_NUMBER: builtins.int
    apm_handle: builtins.int
    data_ptr: builtins.int
    """*mut i16"""
    size: builtins.int
    """in bytes"""
    sample_rate: builtins.int
    num_channels: builtins.int
    def __init__(
        self,
        *,
        apm_handle: builtins.int | None = ...,
        data_ptr: builtins.int | None = ...,
        size: builtins.int | None = ...,
        sample_rate: builtins.int | None = ...,
        num_channels: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["apm_handle", b"apm_handle", "data_ptr", b"data_ptr", "num_channels", b"num_channels", "sample_rate", b"sample_rate", "size", b"size"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["apm_handle", b"apm_handle", "data_ptr", b"data_ptr", "num_channels", b"num_channels", "sample_rate", b"sample_rate", "size", b"size"]) -> None: ...

global___ApmProcessStreamRequest = ApmProcessStreamRequest

@typing.final
class ApmProcessStreamResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ERROR_FIELD_NUMBER: builtins.int
    error: builtins.str
    def __init__(
        self,
        *,
        error: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["error", b"error"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["error", b"error"]) -> None: ...

global___ApmProcessStreamResponse = ApmProcessStreamResponse

@typing.final
class ApmProcessReverseStreamRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    APM_HANDLE_FIELD_NUMBER: builtins.int
    DATA_PTR_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    SAMPLE_RATE_FIELD_NUMBER: builtins.int
    NUM_CHANNELS_FIELD_NUMBER: builtins.int
    apm_handle: builtins.int
    data_ptr: builtins.int
    """*mut i16"""
    size: builtins.int
    """in bytes"""
    sample_rate: builtins.int
    num_channels: builtins.int
    def __init__(
        self,
        *,
        apm_handle: builtins.int | None = ...,
        data_ptr: builtins.int | None = ...,
        size: builtins.int | None = ...,
        sample_rate: builtins.int | None = ...,
        num_channels: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["apm_handle", b"apm_handle", "data_ptr", b"data_ptr", "num_channels", b"num_channels", "sample_rate", b"sample_rate", "size", b"size"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["apm_handle", b"apm_handle", "data_ptr", b"data_ptr", "num_channels", b"num_channels", "sample_rate", b"sample_rate", "size", b"size"]) -> None: ...

global___ApmProcessReverseStreamRequest = ApmProcessReverseStreamRequest

@typing.final
class ApmProcessReverseStreamResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ERROR_FIELD_NUMBER: builtins.int
    error: builtins.str
    def __init__(
        self,
        *,
        error: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["error", b"error"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["error", b"error"]) -> None: ...

global___ApmProcessReverseStreamResponse = ApmProcessReverseStreamResponse

@typing.final
class ApmSetStreamDelayRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    APM_HANDLE_FIELD_NUMBER: builtins.int
    DELAY_MS_FIELD_NUMBER: builtins.int
    apm_handle: builtins.int
    delay_ms: builtins.int
    def __init__(
        self,
        *,
        apm_handle: builtins.int | None = ...,
        delay_ms: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["apm_handle", b"apm_handle", "delay_ms", b"delay_ms"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["apm_handle", b"apm_handle", "delay_ms", b"delay_ms"]) -> None: ...

global___ApmSetStreamDelayRequest = ApmSetStreamDelayRequest

@typing.final
class ApmSetStreamDelayResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ERROR_FIELD_NUMBER: builtins.int
    error: builtins.str
    def __init__(
        self,
        *,
        error: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["error", b"error"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["error", b"error"]) -> None: ...

global___ApmSetStreamDelayResponse = ApmSetStreamDelayResponse

@typing.final
class NewSoxResamplerRequest(google.protobuf.message.Message):
    """New resampler using SoX (much better quality)"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INPUT_RATE_FIELD_NUMBER: builtins.int
    OUTPUT_RATE_FIELD_NUMBER: builtins.int
    NUM_CHANNELS_FIELD_NUMBER: builtins.int
    INPUT_DATA_TYPE_FIELD_NUMBER: builtins.int
    OUTPUT_DATA_TYPE_FIELD_NUMBER: builtins.int
    QUALITY_RECIPE_FIELD_NUMBER: builtins.int
    FLAGS_FIELD_NUMBER: builtins.int
    input_rate: builtins.float
    output_rate: builtins.float
    num_channels: builtins.int
    input_data_type: global___SoxResamplerDataType.ValueType
    output_data_type: global___SoxResamplerDataType.ValueType
    quality_recipe: global___SoxQualityRecipe.ValueType
    flags: builtins.int
    def __init__(
        self,
        *,
        input_rate: builtins.float | None = ...,
        output_rate: builtins.float | None = ...,
        num_channels: builtins.int | None = ...,
        input_data_type: global___SoxResamplerDataType.ValueType | None = ...,
        output_data_type: global___SoxResamplerDataType.ValueType | None = ...,
        quality_recipe: global___SoxQualityRecipe.ValueType | None = ...,
        flags: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["flags", b"flags", "input_data_type", b"input_data_type", "input_rate", b"input_rate", "num_channels", b"num_channels", "output_data_type", b"output_data_type", "output_rate", b"output_rate", "quality_recipe", b"quality_recipe"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["flags", b"flags", "input_data_type", b"input_data_type", "input_rate", b"input_rate", "num_channels", b"num_channels", "output_data_type", b"output_data_type", "output_rate", b"output_rate", "quality_recipe", b"quality_recipe"]) -> None: ...

global___NewSoxResamplerRequest = NewSoxResamplerRequest

@typing.final
class NewSoxResamplerResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESAMPLER_FIELD_NUMBER: builtins.int
    ERROR_FIELD_NUMBER: builtins.int
    error: builtins.str
    @property
    def resampler(self) -> global___OwnedSoxResampler: ...
    def __init__(
        self,
        *,
        resampler: global___OwnedSoxResampler | None = ...,
        error: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["error", b"error", "message", b"message", "resampler", b"resampler"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["error", b"error", "message", b"message", "resampler", b"resampler"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["message", b"message"]) -> typing.Literal["resampler", "error"] | None: ...

global___NewSoxResamplerResponse = NewSoxResamplerResponse

@typing.final
class PushSoxResamplerRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESAMPLER_HANDLE_FIELD_NUMBER: builtins.int
    DATA_PTR_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    resampler_handle: builtins.int
    data_ptr: builtins.int
    """*const i16"""
    size: builtins.int
    """in bytes"""
    def __init__(
        self,
        *,
        resampler_handle: builtins.int | None = ...,
        data_ptr: builtins.int | None = ...,
        size: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["data_ptr", b"data_ptr", "resampler_handle", b"resampler_handle", "size", b"size"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data_ptr", b"data_ptr", "resampler_handle", b"resampler_handle", "size", b"size"]) -> None: ...

global___PushSoxResamplerRequest = PushSoxResamplerRequest

@typing.final
class PushSoxResamplerResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OUTPUT_PTR_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    ERROR_FIELD_NUMBER: builtins.int
    output_ptr: builtins.int
    """*const i16 (could be null)"""
    size: builtins.int
    """in bytes"""
    error: builtins.str
    def __init__(
        self,
        *,
        output_ptr: builtins.int | None = ...,
        size: builtins.int | None = ...,
        error: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["error", b"error", "output_ptr", b"output_ptr", "size", b"size"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["error", b"error", "output_ptr", b"output_ptr", "size", b"size"]) -> None: ...

global___PushSoxResamplerResponse = PushSoxResamplerResponse

@typing.final
class FlushSoxResamplerRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESAMPLER_HANDLE_FIELD_NUMBER: builtins.int
    resampler_handle: builtins.int
    def __init__(
        self,
        *,
        resampler_handle: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["resampler_handle", b"resampler_handle"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["resampler_handle", b"resampler_handle"]) -> None: ...

global___FlushSoxResamplerRequest = FlushSoxResamplerRequest

@typing.final
class FlushSoxResamplerResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OUTPUT_PTR_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    ERROR_FIELD_NUMBER: builtins.int
    output_ptr: builtins.int
    """*const i16 (could be null)"""
    size: builtins.int
    """in bytes"""
    error: builtins.str
    def __init__(
        self,
        *,
        output_ptr: builtins.int | None = ...,
        size: builtins.int | None = ...,
        error: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["error", b"error", "output_ptr", b"output_ptr", "size", b"size"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["error", b"error", "output_ptr", b"output_ptr", "size", b"size"]) -> None: ...

global___FlushSoxResamplerResponse = FlushSoxResamplerResponse

@typing.final
class AudioFrameBufferInfo(google.protobuf.message.Message):
    """
    AudioFrame buffer
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_PTR_FIELD_NUMBER: builtins.int
    NUM_CHANNELS_FIELD_NUMBER: builtins.int
    SAMPLE_RATE_FIELD_NUMBER: builtins.int
    SAMPLES_PER_CHANNEL_FIELD_NUMBER: builtins.int
    data_ptr: builtins.int
    """*const i16"""
    num_channels: builtins.int
    sample_rate: builtins.int
    samples_per_channel: builtins.int
    def __init__(
        self,
        *,
        data_ptr: builtins.int | None = ...,
        num_channels: builtins.int | None = ...,
        sample_rate: builtins.int | None = ...,
        samples_per_channel: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["data_ptr", b"data_ptr", "num_channels", b"num_channels", "sample_rate", b"sample_rate", "samples_per_channel", b"samples_per_channel"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data_ptr", b"data_ptr", "num_channels", b"num_channels", "sample_rate", b"sample_rate", "samples_per_channel", b"samples_per_channel"]) -> None: ...

global___AudioFrameBufferInfo = AudioFrameBufferInfo

@typing.final
class OwnedAudioFrameBuffer(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HANDLE_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    @property
    def handle(self) -> handle_pb2.FfiOwnedHandle: ...
    @property
    def info(self) -> global___AudioFrameBufferInfo: ...
    def __init__(
        self,
        *,
        handle: handle_pb2.FfiOwnedHandle | None = ...,
        info: global___AudioFrameBufferInfo | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["handle", b"handle", "info", b"info"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["handle", b"handle", "info", b"info"]) -> None: ...

global___OwnedAudioFrameBuffer = OwnedAudioFrameBuffer

@typing.final
class AudioStreamInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_FIELD_NUMBER: builtins.int
    type: global___AudioStreamType.ValueType
    def __init__(
        self,
        *,
        type: global___AudioStreamType.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["type", b"type"]) -> None: ...

global___AudioStreamInfo = AudioStreamInfo

@typing.final
class OwnedAudioStream(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HANDLE_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    @property
    def handle(self) -> handle_pb2.FfiOwnedHandle: ...
    @property
    def info(self) -> global___AudioStreamInfo: ...
    def __init__(
        self,
        *,
        handle: handle_pb2.FfiOwnedHandle | None = ...,
        info: global___AudioStreamInfo | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["handle", b"handle", "info", b"info"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["handle", b"handle", "info", b"info"]) -> None: ...

global___OwnedAudioStream = OwnedAudioStream

@typing.final
class AudioStreamEvent(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STREAM_HANDLE_FIELD_NUMBER: builtins.int
    FRAME_RECEIVED_FIELD_NUMBER: builtins.int
    EOS_FIELD_NUMBER: builtins.int
    stream_handle: builtins.int
    @property
    def frame_received(self) -> global___AudioFrameReceived: ...
    @property
    def eos(self) -> global___AudioStreamEOS: ...
    def __init__(
        self,
        *,
        stream_handle: builtins.int | None = ...,
        frame_received: global___AudioFrameReceived | None = ...,
        eos: global___AudioStreamEOS | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["eos", b"eos", "frame_received", b"frame_received", "message", b"message", "stream_handle", b"stream_handle"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["eos", b"eos", "frame_received", b"frame_received", "message", b"message", "stream_handle", b"stream_handle"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["message", b"message"]) -> typing.Literal["frame_received", "eos"] | None: ...

global___AudioStreamEvent = AudioStreamEvent

@typing.final
class AudioFrameReceived(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FRAME_FIELD_NUMBER: builtins.int
    @property
    def frame(self) -> global___OwnedAudioFrameBuffer: ...
    def __init__(
        self,
        *,
        frame: global___OwnedAudioFrameBuffer | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["frame", b"frame"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["frame", b"frame"]) -> None: ...

global___AudioFrameReceived = AudioFrameReceived

@typing.final
class AudioStreamEOS(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___AudioStreamEOS = AudioStreamEOS

@typing.final
class AudioSourceOptions(google.protobuf.message.Message):
    """
    AudioSource
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ECHO_CANCELLATION_FIELD_NUMBER: builtins.int
    NOISE_SUPPRESSION_FIELD_NUMBER: builtins.int
    AUTO_GAIN_CONTROL_FIELD_NUMBER: builtins.int
    echo_cancellation: builtins.bool
    noise_suppression: builtins.bool
    auto_gain_control: builtins.bool
    def __init__(
        self,
        *,
        echo_cancellation: builtins.bool | None = ...,
        noise_suppression: builtins.bool | None = ...,
        auto_gain_control: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["auto_gain_control", b"auto_gain_control", "echo_cancellation", b"echo_cancellation", "noise_suppression", b"noise_suppression"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["auto_gain_control", b"auto_gain_control", "echo_cancellation", b"echo_cancellation", "noise_suppression", b"noise_suppression"]) -> None: ...

global___AudioSourceOptions = AudioSourceOptions

@typing.final
class AudioSourceInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_FIELD_NUMBER: builtins.int
    type: global___AudioSourceType.ValueType
    def __init__(
        self,
        *,
        type: global___AudioSourceType.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["type", b"type"]) -> None: ...

global___AudioSourceInfo = AudioSourceInfo

@typing.final
class OwnedAudioSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HANDLE_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    @property
    def handle(self) -> handle_pb2.FfiOwnedHandle: ...
    @property
    def info(self) -> global___AudioSourceInfo: ...
    def __init__(
        self,
        *,
        handle: handle_pb2.FfiOwnedHandle | None = ...,
        info: global___AudioSourceInfo | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["handle", b"handle", "info", b"info"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["handle", b"handle", "info", b"info"]) -> None: ...

global___OwnedAudioSource = OwnedAudioSource

@typing.final
class AudioResamplerInfo(google.protobuf.message.Message):
    """
    AudioResampler
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___AudioResamplerInfo = AudioResamplerInfo

@typing.final
class OwnedAudioResampler(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HANDLE_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    @property
    def handle(self) -> handle_pb2.FfiOwnedHandle: ...
    @property
    def info(self) -> global___AudioResamplerInfo: ...
    def __init__(
        self,
        *,
        handle: handle_pb2.FfiOwnedHandle | None = ...,
        info: global___AudioResamplerInfo | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["handle", b"handle", "info", b"info"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["handle", b"handle", "info", b"info"]) -> None: ...

global___OwnedAudioResampler = OwnedAudioResampler

@typing.final
class OwnedApm(google.protobuf.message.Message):
    """
    AEC
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HANDLE_FIELD_NUMBER: builtins.int
    @property
    def handle(self) -> handle_pb2.FfiOwnedHandle: ...
    def __init__(
        self,
        *,
        handle: handle_pb2.FfiOwnedHandle | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["handle", b"handle"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["handle", b"handle"]) -> None: ...

global___OwnedApm = OwnedApm

@typing.final
class SoxResamplerInfo(google.protobuf.message.Message):
    """
    Sox AudioResampler
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___SoxResamplerInfo = SoxResamplerInfo

@typing.final
class OwnedSoxResampler(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HANDLE_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    @property
    def handle(self) -> handle_pb2.FfiOwnedHandle: ...
    @property
    def info(self) -> global___SoxResamplerInfo: ...
    def __init__(
        self,
        *,
        handle: handle_pb2.FfiOwnedHandle | None = ...,
        info: global___SoxResamplerInfo | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["handle", b"handle", "info", b"info"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["handle", b"handle", "info", b"info"]) -> None: ...

global___OwnedSoxResampler = OwnedSoxResampler

@typing.final
class LoadAudioFilterPluginRequest(google.protobuf.message.Message):
    """Audio Filter Plugin"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PLUGIN_PATH_FIELD_NUMBER: builtins.int
    DEPENDENCIES_FIELD_NUMBER: builtins.int
    MODULE_ID_FIELD_NUMBER: builtins.int
    plugin_path: builtins.str
    """path for ffi audio filter plugin"""
    module_id: builtins.str
    """Unique identifier of the plugin"""
    @property
    def dependencies(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Optional: paths for dependency dylibs"""

    def __init__(
        self,
        *,
        plugin_path: builtins.str | None = ...,
        dependencies: collections.abc.Iterable[builtins.str] | None = ...,
        module_id: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["module_id", b"module_id", "plugin_path", b"plugin_path"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dependencies", b"dependencies", "module_id", b"module_id", "plugin_path", b"plugin_path"]) -> None: ...

global___LoadAudioFilterPluginRequest = LoadAudioFilterPluginRequest

@typing.final
class LoadAudioFilterPluginResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ERROR_FIELD_NUMBER: builtins.int
    error: builtins.str
    def __init__(
        self,
        *,
        error: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["error", b"error"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["error", b"error"]) -> None: ...

global___LoadAudioFilterPluginResponse = LoadAudioFilterPluginResponse



# Copyright 2023 LiveKit, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Integration test for ParticipantRecorder with live streams.

This test connects to an actual LiveKit room and records from a live participant
stream. It validates the recorded output to ensure the recording implementation is correct.

Usage:
    # Set environment variables
    export LIVEKIT_URL=wss://your-livekit-server.com
    export LIVEKIT_API_KEY=your-api-key
    export LIVEKIT_API_SECRET=your-api-secret
    
    # Run the test
    python -m pytest tests/test_recorder_integration.py -v -s
    
    # Or run directly
    python tests/test_recorder_integration.py
"""

import asyncio
import os
import sys
import time
import tempfile
import logging
from pathlib import Path
from typing import Optional, Any, Dict

# Add parent directory to path for direct imports
sys.path.insert(0, str(Path(__file__).parent.parent))
# Add livekit-api and livekit-protocol directories to path for API imports
sys.path.insert(0, str(Path(__file__).parent.parent.parent / "livekit-protocol"))
sys.path.insert(0, str(Path(__file__).parent.parent.parent / "livekit-api"))

# Direct imports to avoid version conflicts with pip-installed livekit
from livekit import rtc
from livekit import api as livekit_api
from livekit.rtc.recorder import (
    ParticipantRecorder,
    RecordingStats,
    ParticipantNotFoundError,
    RecordingError,
)

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
)
logger = logging.getLogger(__name__)

# Import pump.fun service for automatic stream discovery
try:
    from .pumpfun_service import PumpFunService
    HAS_PUMPFUN = True
except ImportError:
    try:
        from pumpfun_service import PumpFunService
        HAS_PUMPFUN = True
    except ImportError:
        HAS_PUMPFUN = False
        logger.warning("PumpFunService not available. Install httpx for automatic stream discovery.")

# For WebM validation
try:
    import av
    HAS_AV = True
except ImportError:
    HAS_AV = False
    print("WARNING: PyAV not available. WebM validation will be skipped.")


class RecordingValidator:
    """Validates recorded WebM files."""

    def __init__(self, file_path: str) -> None:
        self.file_path = file_path

    def validate(self) -> dict[str, Any]:
        """Validate the recorded WebM file.

        Returns:
            Dictionary with validation results.
        """
        if not HAS_AV:
            return {"valid": False, "error": "PyAV not available"}

        if not os.path.exists(self.file_path):
            return {"valid": False, "error": "File does not exist"}

        file_size = os.path.getsize(self.file_path)
        if file_size == 0:
            return {"valid": False, "error": "File is empty"}

        try:
            container = av.open(self.file_path)
            streams_info = []

            for stream in container.streams:
                stream_info = {
                    "type": str(stream.type),
                    "codec_name": stream.codec.name if stream.codec else None,
                    "duration": float(stream.duration * stream.time_base) if stream.duration else None,
                    "bitrate": stream.bitrate if hasattr(stream, "bitrate") else None,
                }

                if stream.type == "video":
                    # Get framerate from codec_context or calculate from average_rate
                    framerate = None
                    if hasattr(stream, "average_rate") and stream.average_rate:
                        framerate = float(stream.average_rate)
                    elif hasattr(stream.codec_context, "framerate") and stream.codec_context.framerate:
                        rate = stream.codec_context.framerate
                        framerate = float(rate.numerator) / float(rate.denominator) if rate.denominator else None
                    
                    # Get pixel format from codec context or first frame
                    pixel_format = None
                    if hasattr(stream.codec_context, "pix_fmt") and stream.codec_context.pix_fmt:
                        pixel_format = str(stream.codec_context.pix_fmt)
                    
                    # Get time base for PTS calculation validation
                    time_base = None
                    if stream.time_base:
                        time_base = {
                            "numerator": stream.time_base.numerator,
                            "denominator": stream.time_base.denominator,
                            "value": float(stream.time_base),
                        }
                    
                    # Calculate expected duration from frame count and frame rate
                    # This helps validate PTS calculation accuracy
                    calculated_duration = None
                    if framerate and framerate > 0:
                        frame_counts = self._count_frames()
                        video_frame_count = frame_counts.get("video_frames", 0)
                        if video_frame_count > 0:
                            calculated_duration = video_frame_count / framerate
                    
                    stream_info.update({
                        "width": stream.width,
                        "height": stream.height,
                        "framerate": framerate,
                        "pixel_format": pixel_format,
                        "time_base": time_base,
                        "calculated_duration_from_frames": calculated_duration,
                    })

                if stream.type == "audio":
                    # Determine audio layout
                    layout = None
                    if stream.channels == 1:
                        layout = "mono"
                    elif stream.channels == 2:
                        layout = "stereo"
                    else:
                        layout = f"{stream.channels}ch"
                    
                    # Get sample format if available
                    sample_format = None
                    if hasattr(stream.codec_context, "sample_fmt") and stream.codec_context.sample_fmt:
                        sample_format = str(stream.codec_context.sample_fmt)
                    
                    # Get time base for audio PTS calculation validation
                    audio_time_base = None
                    if stream.time_base:
                        audio_time_base = {
                            "numerator": stream.time_base.numerator,
                            "denominator": stream.time_base.denominator,
                            "value": float(stream.time_base),
                        }
                    elif stream.rate and stream.rate > 0:
                        # Audio time_base is typically 1/sample_rate
                        audio_time_base = {
                            "numerator": 1,
                            "denominator": stream.rate,
                            "value": 1.0 / stream.rate,
                        }
                    
                    stream_info.update({
                        "sample_rate": stream.rate,
                        "channels": stream.channels,
                        "layout": layout,
                        "sample_format": sample_format,
                        "time_base": audio_time_base,
                    })

                streams_info.append(stream_info)

            container.close()

            # Count frames by seeking through the file
            frame_counts = self._count_frames()

            # Calculate timing validation metrics
            timing_validation = {}
            video_stream = next((s for s in streams_info if s.get("type") == "video"), None)
            audio_stream = next((s for s in streams_info if s.get("type") == "audio"), None)
            
            if video_stream:
                container_duration = video_stream.get("duration")
                calculated_duration = video_stream.get("calculated_duration_from_frames")
                framerate = video_stream.get("framerate")
                
                timing_validation["video"] = {
                    "container_duration": container_duration,
                    "calculated_duration_from_frames": calculated_duration,
                    "framerate": framerate,
                    "duration_match": None,
                }
                
                # Check if container duration matches calculated duration
                if container_duration and calculated_duration:
                    duration_diff = abs(container_duration - calculated_duration)
                    duration_tolerance = 0.1  # 100ms tolerance
                    timing_validation["video"]["duration_match"] = duration_diff < duration_tolerance
                    timing_validation["video"]["duration_diff_seconds"] = duration_diff
            
            return {
                "valid": True,
                "file_size_bytes": file_size,
                "streams": streams_info,
                "frame_counts": frame_counts,
                "timing_validation": timing_validation,
            }
        except Exception as e:
            return {"valid": False, "error": str(e), "exception": e}

    def _count_frames(self) -> dict[str, int]:
        """Count frames in the video file."""
        if not HAS_AV:
            return {}

        try:
            container = av.open(self.file_path)
            video_frames = 0
            audio_frames = 0

            for packet in container.demux():
                for frame in packet.decode():
                    if isinstance(frame, av.VideoFrame):
                        video_frames += 1
                    elif isinstance(frame, av.AudioFrame):
                        audio_frames += 1

            container.close()

            return {
                "video_frames": video_frames,
                "audio_frames": audio_frames,
            }
        except Exception as e:
            logger.warning(f"Error counting frames: {e}")
            return {}


async def wait_for_participant(
    room: rtc.Room,
    participant_identity: Optional[str] = None,
    timeout: float = 30.0,
) -> rtc.RemoteParticipant:
    """Wait for a remote participant to join the room.

    Args:
        room: The LiveKit room.
        participant_identity: Optional specific participant identity to wait for.
        timeout: Maximum time to wait in seconds.

    Returns:
        The remote participant.

    Raises:
        TimeoutError: If no participant joins within the timeout.
    """
    start_time = time.time()
    participant_event = asyncio.Event()
    found_participant: Optional[rtc.RemoteParticipant] = None

    def on_participant_connected(participant: rtc.RemoteParticipant) -> None:
        nonlocal found_participant
        if participant_identity is None or participant.identity == participant_identity:
            found_participant = participant
            if not participant_event.is_set():
                participant_event.set()

    room.on("participant_connected", on_participant_connected)

    # Check if participant already exists
    if participant_identity:
        if participant_identity in room.remote_participants:
            found_participant = room.remote_participants[participant_identity]
            participant_event.set()
    else:
        if room.remote_participants:
            found_participant = list(room.remote_participants.values())[0]
            participant_event.set()

    # Wait for participant if not already found
    if found_participant is None:
        try:
            await asyncio.wait_for(participant_event.wait(), timeout=timeout)
        except asyncio.TimeoutError:
            raise TimeoutError(
                f"No participant joined within {timeout} seconds"
            )

    if found_participant is None:
        raise ValueError("Participant not found")

    # Wait a bit for tracks to be published
    await asyncio.sleep(2.0)

    return found_participant


async def wait_for_tracks(
    participant: rtc.RemoteParticipant,
    timeout: float = 10.0,
) -> tuple[bool, bool]:
    """Wait for participant to publish video and/or audio tracks.

    Args:
        participant: The remote participant.
        timeout: Maximum time to wait in seconds.

    Returns:
        Tuple of (has_video, has_audio).
    """
    start_time = time.time()
    has_video = False
    has_audio = False

    # Check existing tracks
    for publication in participant.track_publications.values():
        if publication.kind == rtc.TrackKind.KIND_VIDEO:
            has_video = True
        elif publication.kind == rtc.TrackKind.KIND_AUDIO:
            has_audio = True

    if has_video and has_audio:
        return (has_video, has_audio)

    # Wait for tracks to be published
    track_event = asyncio.Event()

    def on_track_published(
        publication: rtc.RemoteTrackPublication,
        p: rtc.RemoteParticipant,
    ) -> None:
        nonlocal has_video, has_audio
        if p.identity != participant.identity:
            return

        if publication.kind == rtc.TrackKind.KIND_VIDEO:
            has_video = True
        elif publication.kind == rtc.TrackKind.KIND_AUDIO:
            has_audio = True

        if has_video or has_audio:
            track_event.set()

    # Note: In a real scenario, you'd register this on the room
    # For testing, we'll poll
    while time.time() - start_time < timeout:
        for publication in participant.track_publications.values():
            if publication.kind == rtc.TrackKind.KIND_VIDEO:
                has_video = True
            elif publication.kind == rtc.TrackKind.KIND_AUDIO:
                has_audio = True

        if has_video or has_audio:
            break

        await asyncio.sleep(0.5)

    return (has_video, has_audio)


async def integration_test_recording() -> dict[str, Any]:
    """Run integration test for recording.

    Automatically discovers a live stream from pump.fun if no credentials provided,
    otherwise uses the provided LiveKit credentials.

    Returns:
        Dictionary with test results.
    """
    # Check if using pump.fun or custom LiveKit server
    use_pumpfun = os.getenv("LIVEKIT_USE_PUMPFUN", "true").lower() == "true"
    
    livekit_url = os.getenv("LIVEKIT_URL")
    livekit_api_key = os.getenv("LIVEKIT_API_KEY")
    livekit_api_secret = os.getenv("LIVEKIT_API_SECRET")
    room_name = os.getenv("LIVEKIT_ROOM_NAME")
    participant_identity = os.getenv("LIVEKIT_PARTICIPANT_IDENTITY")  # Optional
    recording_duration = float(os.getenv("LIVEKIT_RECORDING_DURATION", "5.0"))

    # Use pump.fun if enabled and no custom credentials provided
    pumpfun_service: Optional[PumpFunService] = None
    stream_info: Optional[Dict[str, Any]] = None
    mint_id: Optional[str] = None
    use_token_directly = False
    custom_token: Optional[str] = None

    if (use_pumpfun and HAS_PUMPFUN and 
        (not livekit_url or not livekit_api_key or not livekit_api_secret)):
        logger.info("Using pump.fun for automatic stream discovery...")
        try:
            pumpfun_service = PumpFunService()
            livekit_url = pumpfun_service.get_livekit_url()
            
            # Get a random live stream
            stream_info = await pumpfun_service.get_random_live_stream(
                min_participants=1,  # At least 1 participant (the streamer)
                exclude_nsfw=True,  # Exclude NSFW for testing
            )
            
            if not stream_info:
                return {
                    "success": False,
                    "error": "No live streams found on pump.fun. Try again later.",
                    "source": "pumpfun",
                }
            
            mint_id = stream_info.get("mint")
            if not mint_id:
                return {
                    "success": False,
                    "error": "Selected stream has no mint ID",
                    "source": "pumpfun",
                }
            
            # Get token for the stream
            logger.info(f"Getting token for stream: {stream_info.get('name')} ({mint_id})")
            token = await pumpfun_service.get_livestream_token(mint_id, role="viewer")
            
            if not token:
                return {
                    "success": False,
                    "error": f"Failed to get token for stream {mint_id}",
                    "source": "pumpfun",
                    "mint_id": mint_id,
                }
            
            logger.info(f"Successfully obtained token for pump.fun stream")
            results = {
                "success": False,
                "source": "pumpfun",
                "stream_info": {
                    "mint_id": mint_id,
                    "name": stream_info.get("name"),
                    "symbol": stream_info.get("symbol"),
                    "participants": stream_info.get("num_participants", 0),
                },
                "errors": [],
                "warnings": [],
            }
            
            # Use the token directly (it's already a JWT)
            use_token_directly = True
            custom_token = token
            
        except Exception as e:
            logger.error(f"Error setting up pump.fun: {e}", exc_info=True)
            return {
                "success": False,
                "error": f"Failed to set up pump.fun service: {e}",
                "source": "pumpfun",
            }
    else:
        # Use custom LiveKit server
        if not livekit_url or not livekit_api_key or not livekit_api_secret:
            return {
                "success": False,
                "error": (
                    "Missing required environment variables: "
                    "LIVEKIT_URL, LIVEKIT_API_KEY, LIVEKIT_API_SECRET. "
                    "Or set LIVEKIT_USE_PUMPFUN=false and provide custom credentials."
                ),
            }
        
        use_token_directly = False
        custom_token = None
        results = {
            "success": False,
            "source": "custom",
            "errors": [],
            "warnings": [],
        }

    if not HAS_AV:
        return {
            "success": False,
            "error": "PyAV is required for recording. Install with: pip install av",
        }

    results = {
        "success": False,
        "room_name": room_name,
        "participant_identity": participant_identity,
        "recording_duration_seconds": recording_duration,
        "errors": [],
        "warnings": [],
    }

    room = rtc.Room()
    recorder: Optional[ParticipantRecorder] = None
    output_file: Optional[str] = None

    try:
        # Create access token
        if use_token_directly:
            # Use the token from pump.fun directly
            token = custom_token
            logger.info("Using pump.fun provided token")
        else:
            # Create token for custom LiveKit server
            token = (
                livekit_api.AccessToken(livekit_api_key, livekit_api_secret)
                .with_identity("recorder-test-agent")
                .with_name("Recording Test Agent")
                .with_grants(
                    livekit_api.VideoGrants(
                        room_join=True,
                        room=room_name or "test-recording-room",
                        room_record=True,
                    )
                )
                .to_jwt()
            )

        logger.info(f"Connecting to room: {room_name or 'pump.fun stream'}")
        try:
            # Add timeout to prevent hanging indefinitely
            await asyncio.wait_for(
                room.connect(livekit_url, token),
                timeout=30.0  # 30 second timeout for connection
            )
        except asyncio.TimeoutError:
            error_msg = "Room connection timed out after 30 seconds - FFI server may be in bad state"
            logger.error(error_msg)
            raise rtc.ConnectError(error_msg)
        except Exception as e:
            logger.error(f"Error connecting to room: {e}")
            raise
        
        results["connected"] = True
        results["room_name"] = room.name

        logger.info("Waiting for participant to join...")
        participant = await wait_for_participant(
            room,
            participant_identity=participant_identity,
            timeout=30.0,
        )
        results["participant_found"] = True
        results["participant_identity"] = participant.identity
        results["participant_sid"] = participant.sid

        logger.info(f"Participant found: {participant.identity}")

        # Wait for tracks with timeout to prevent hanging
        try:
            has_video, has_audio = await asyncio.wait_for(
                wait_for_tracks(participant, timeout=20.0),
                timeout=25.0  # Overall timeout including wait_for_tracks internal timeout
            )
        except asyncio.TimeoutError:
            logger.warning("Timeout waiting for tracks, proceeding with no tracks")
            has_video = False
            has_audio = False
        results["has_video"] = has_video
        results["has_audio"] = has_audio

        if not has_video and not has_audio:
            results["warnings"].append("No video or audio tracks found")

        logger.info(f"Tracks available - Video: {has_video}, Audio: {has_audio}")

        # Create recorder
        # Use VP9 with best quality for highest video quality
        recorder = ParticipantRecorder(
            room,
            video_codec="vp9",
            video_quality="best",
            auto_bitrate=True,
        )
        results["recorder_created"] = True

        # Start recording with timeout
        logger.info(f"Starting recording for {recording_duration} seconds...")
        try:
            await asyncio.wait_for(
                recorder.start_recording(participant.identity),
                timeout=10.0
            )
        except asyncio.TimeoutError:
            raise RecordingError("Timeout starting recording - participant may have disconnected")
        results["recording_started"] = True
        results["recording_start_time"] = time.time()

        # Record for specified duration
        await asyncio.sleep(recording_duration)

        # Get stats during recording
        stats = recorder.get_stats()
        results["stats_during_recording"] = {
            "video_frames_recorded": stats.video_frames_recorded,
            "audio_frames_recorded": stats.audio_frames_recorded,
            "recording_duration_seconds": stats.recording_duration_seconds,
        }

        logger.info(f"Recording stats: {results['stats_during_recording']}")

        # Stop recording and save
        with tempfile.NamedTemporaryFile(suffix=".webm", delete=False) as tmp:
            output_file = tmp.name

        logger.info(f"Stopping recording and saving to: {output_file}")
        saved_path = await recorder.stop_recording(output_file)
        results["recording_stopped"] = True
        results["output_file"] = saved_path

        # Get final stats
        final_stats = recorder.get_stats()
        results["final_stats"] = {
            "video_frames_recorded": final_stats.video_frames_recorded,
            "audio_frames_recorded": final_stats.audio_frames_recorded,
            "recording_duration_seconds": final_stats.recording_duration_seconds,
            "output_file_size_bytes": final_stats.output_file_size_bytes,
        }

        logger.info(f"Final stats: {results['final_stats']}")

        # Validate output file
        logger.info("Validating recorded file...")
        validator = RecordingValidator(saved_path)
        validation_result = validator.validate()
        results["validation"] = validation_result

        if validation_result.get("valid"):
            logger.info("✓ Recording validation passed")
            logger.info(f"  File size: {validation_result.get('file_size_bytes')} bytes")
            logger.info(f"  Streams: {len(validation_result.get('streams', []))}")

            for stream_info in validation_result.get("streams", []):
                stream_type = stream_info.get("type")
                codec = stream_info.get("codec_name")
                logger.info(f"  - {stream_type} stream: {codec}")

            frame_counts = validation_result.get("frame_counts", {})
            if frame_counts:
                logger.info(f"  - Video frames: {frame_counts.get('video_frames', 0)}")
                logger.info(f"  - Audio frames: {frame_counts.get('audio_frames', 0)}")
        else:
            logger.error(f"✗ Recording validation failed: {validation_result.get('error')}")
            results["errors"].append(f"Validation failed: {validation_result.get('error')}")

        # Basic checks
        if final_stats.video_frames_recorded == 0 and has_video:
            results["warnings"].append("No video frames were recorded despite video track being available")

        if final_stats.audio_frames_recorded == 0 and has_audio:
            results["warnings"].append("No audio frames were recorded despite audio track being available")

        if final_stats.output_file_size_bytes == 0:
            results["errors"].append("Output file is empty")

        results["success"] = (
            len(results["errors"]) == 0
            and validation_result.get("valid", False)
            and final_stats.output_file_size_bytes > 0
        )

    except rtc.ConnectError as e:
        error_msg = f"Connection error: {e}"
        logger.error(error_msg)
        results["errors"].append(error_msg)
        results["success"] = False
    except ParticipantNotFoundError as e:
        error_msg = f"Participant not found: {e}"
        logger.error(error_msg)
        results["errors"].append(error_msg)
        results["success"] = False
    except RecordingError as e:
        error_msg = f"Recording error: {e}"
        logger.error(error_msg)
        results["errors"].append(error_msg)
        results["success"] = False
    except TimeoutError as e:
        error_msg = f"Timeout: {e}"
        logger.error(error_msg)
        results["errors"].append(error_msg)
        results["success"] = False
    except Exception as e:
        error_msg = f"Unexpected error: {e}"
        logger.error(error_msg, exc_info=True)
        results["errors"].append(error_msg)
        results["success"] = False
    finally:
        # Cleanup - ensure all resources are released
        if recorder:
            try:
                if recorder.is_recording:
                    await recorder.stop_recording(output_file or "cleanup.webm")
            except Exception:
                pass
            finally:
                # Explicitly clear recorder reference
                recorder = None

        if room:
            try:
                # Wait for disconnect to complete fully with timeout
                await asyncio.wait_for(room.disconnect(), timeout=5.0)
                # Give async cleanup a moment to complete
                await asyncio.sleep(0.2)
            except asyncio.TimeoutError:
                logger.warning("Room disconnect timed out, forcing cleanup")
            except Exception as e:
                logger.warning(f"Error during room disconnect: {e}")
            finally:
                # Explicitly clear room reference
                room = None

        if pumpfun_service:
            try:
                await pumpfun_service.close()
            except Exception:
                pass
            finally:
                # Explicitly clear pumpfun_service reference
                pumpfun_service = None

        # Force garbage collection to ensure all references are released
        import gc
        gc.collect()

    return results


async def main() -> None:
    """Main entry point for running the integration test."""
    print("=" * 80)
    print("LiveKit ParticipantRecorder Integration Test")
    print("=" * 80)
    print()

    results = await integration_test_recording()

    print()
    print("=" * 80)
    print("Test Results")
    print("=" * 80)
    print(f"Success: {results.get('success', False)}")
    print(f"Source: {results.get('source', 'unknown')}")
    
    if results.get('stream_info'):
        stream_info = results['stream_info']
        print(f"Stream: {stream_info.get('name', 'Unknown')} ({stream_info.get('symbol', 'N/A')})")
        print(f"  Mint ID: {stream_info.get('mint_id')}")
        print(f"  Participants: {stream_info.get('participants', 0)}")
    
    print(f"Room: {results.get('room_name', 'N/A')}")
    print(f"Participant: {results.get('participant_identity', 'N/A')}")

    if results.get("errors"):
        print("\nErrors:")
        for error in results["errors"]:
            print(f"  - {error}")

    if results.get("warnings"):
        print("\nWarnings:")
        for warning in results["warnings"]:
            print(f"  - {warning}")

    if results.get("final_stats"):
        stats = results["final_stats"]
        print("\nFinal Statistics:")
        print(f"  Video frames: {stats.get('video_frames_recorded', 0)}")
        print(f"  Audio frames: {stats.get('audio_frames_recorded', 0)}")
        print(f"  Duration: {stats.get('recording_duration_seconds', 0):.2f}s")
        print(f"  File size: {stats.get('output_file_size_bytes', 0)} bytes")

    if results.get("validation"):
        validation = results["validation"]
        if validation.get("valid"):
            print("\nValidation: PASSED")
            print(f"  File size: {validation.get('file_size_bytes')} bytes")
            print(f"  Streams: {len(validation.get('streams', []))}")
            for stream in validation.get("streams", []):
                stream_type = stream.get("type")
                codec = stream.get("codec_name")
                print(f"    - {stream_type}: {codec}")
        else:
            print("\nValidation: FAILED")
            print(f"  Error: {validation.get('error')}")

    if results.get("output_file"):
        print(f"\nOutput file: {results['output_file']}")

    print()
    print("=" * 80)

    # Exit with appropriate code
    sys.exit(0 if results.get("success") else 1)


# Pytest-compatible test function
def test_recorder_integration() -> None:
    """Pytest-compatible integration test."""
    results = asyncio.run(integration_test_recording())
    assert results.get("success", False), f"Integration test failed: {results.get('errors', [])}"


if __name__ == "__main__":
    asyncio.run(main())




# Copyright 2023 LiveKit, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Optional, Dict, Union, ClassVar
from enum import IntEnum
from ._proto import rpc_pb2 as proto_rpc
from dataclasses import dataclass


@dataclass
class RpcInvocationData:
    """Data passed to method handler for incoming RPC invocations

    Attributes:
        request_id (str): The unique request ID. Will match at both sides of the call, useful for debugging or logging.
        caller_identity (str): The unique participant identity of the caller.
        payload (str): The payload of the request. User-definable format, typically JSON.
        response_timeout (float): The maximum time the caller will wait for a response.
    """

    request_id: str
    caller_identity: str
    payload: str
    response_timeout: float


class RpcError(Exception):
    """
    Specialized error handling for RPC methods.

    Instances of this type, when thrown in a method handler, will have their `message`
    serialized and sent across the wire. The caller will receive an equivalent error on the other side.

    Built-in errors are included (codes 1001-1999) but developers may use the code, message, and data fields to create their own errors.
    """

    class ErrorCode(IntEnum):
        APPLICATION_ERROR = 1500
        CONNECTION_TIMEOUT = 1501
        RESPONSE_TIMEOUT = 1502
        RECIPIENT_DISCONNECTED = 1503
        RESPONSE_PAYLOAD_TOO_LARGE = 1504
        SEND_FAILED = 1505

        UNSUPPORTED_METHOD = 1400
        RECIPIENT_NOT_FOUND = 1401
        REQUEST_PAYLOAD_TOO_LARGE = 1402
        UNSUPPORTED_SERVER = 1403
        UNSUPPORTED_VERSION = 1404

    ErrorMessage: ClassVar[Dict[ErrorCode, str]] = {
        ErrorCode.APPLICATION_ERROR: "Application error in method handler",
        ErrorCode.CONNECTION_TIMEOUT: "Connection timeout",
        ErrorCode.RESPONSE_TIMEOUT: "Response timeout",
        ErrorCode.RECIPIENT_DISCONNECTED: "Recipient disconnected",
        ErrorCode.RESPONSE_PAYLOAD_TOO_LARGE: "Response payload too large",
        ErrorCode.SEND_FAILED: "Failed to send",
        ErrorCode.UNSUPPORTED_METHOD: "Method not supported at destination",
        ErrorCode.RECIPIENT_NOT_FOUND: "Recipient not found",
        ErrorCode.REQUEST_PAYLOAD_TOO_LARGE: "Request payload too large",
        ErrorCode.UNSUPPORTED_SERVER: "RPC not supported by server",
        ErrorCode.UNSUPPORTED_VERSION: "Unsupported RPC version",
    }

    def __init__(
        self,
        code: Union[int, "RpcError.ErrorCode"],
        message: str,
        data: Optional[str] = None,
    ):
        """
        Creates an error object with the given code and message, plus an optional data payload.

        If thrown in an RPC method handler, the error will be sent back to the caller.

        Args:
            code (int): Your error code (Error codes 1001-1999 are reserved for built-in errors)
            message (str): A readable error message.
            data (Optional[str]): Optional additional data associated with the error (JSON recommended)
        """
        super().__init__(message)
        self._code = code
        self._message = message
        self._data = data

    @property
    def code(self) -> int:
        """Error code value. Codes 1001-1999 are reserved for built-in errors (see RpcError.ErrorCode for their meanings)."""
        return self._code

    @property
    def message(self) -> str:
        """A readable error message."""
        return self._message

    @property
    def data(self) -> Optional[str]:
        """Optional additional data associated with the error (JSON recommended)."""
        return self._data

    @classmethod
    def _from_proto(cls, proto: proto_rpc.RpcError) -> "RpcError":
        return cls(proto.code, proto.message, proto.data)

    def _to_proto(self) -> proto_rpc.RpcError:
        return proto_rpc.RpcError(code=self.code, message=self.message, data=self.data)

    @classmethod
    def _built_in(cls, code: "RpcError.ErrorCode", data: Optional[str] = None) -> "RpcError":
        message = cls.ErrorMessage[code]
        return cls(code, message, data)



from __future__ import annotations

import ctypes
from enum import Enum, unique

from ._proto import audio_frame_pb2 as proto_audio_frame
from ._ffi_client import FfiClient, FfiHandle
from ._proto import ffi_pb2 as proto_ffi
from ._utils import get_address
from .audio_frame import AudioFrame


@unique
class AudioResamplerQuality(str, Enum):
    QUICK = "quick"
    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"
    VERY_HIGH = "very_high"


class AudioResampler:
    """
    A class for resampling audio data from one sample rate to another.

    `AudioResampler` provides functionality to resample audio data from an input sample rate to an output
    sample rate using the Sox resampling library. It supports multiple channels and configurable resampling quality.
    """

    def __init__(
        self,
        input_rate: int,
        output_rate: int,
        *,
        num_channels: int = 1,
        quality: AudioResamplerQuality = AudioResamplerQuality.MEDIUM,
    ) -> None:
        """
        Initialize an `AudioResampler` instance for resampling audio data.

        Args:
            input_rate (int): The sample rate of the input audio data (in Hz).
            output_rate (int): The desired sample rate of the output audio data (in Hz).
            num_channels (int, optional): The number of audio channels (e.g., 1 for mono, 2 for stereo). Defaults to 1.
            quality (AudioResamplerQuality, optional): The quality setting for the resampler. Can be one of the
                `AudioResamplerQuality` enum values: `QUICK`, `LOW`, `MEDIUM`, `HIGH`, `VERY_HIGH`. Higher quality settings
                result in better audio quality but require more processing power. Defaults to `AudioResamplerQuality.MEDIUM`.

        Raises:
            Exception: If there is an error creating the resampler.
        """
        self._input_rate = input_rate
        self._output_rate = output_rate
        self._num_channels = num_channels

        req = proto_ffi.FfiRequest()
        req.new_sox_resampler.input_rate = input_rate
        req.new_sox_resampler.output_rate = output_rate
        req.new_sox_resampler.num_channels = num_channels
        req.new_sox_resampler.quality_recipe = _to_proto_quality(quality)

        # not exposed for now
        req.new_sox_resampler.input_data_type = (
            proto_audio_frame.SoxResamplerDataType.SOXR_DATATYPE_INT16I
        )
        req.new_sox_resampler.output_data_type = (
            proto_audio_frame.SoxResamplerDataType.SOXR_DATATYPE_INT16I
        )
        req.new_sox_resampler.flags = 0  # default

        resp = FfiClient.instance.request(req)

        if resp.new_sox_resampler.error:
            raise Exception(resp.new_sox_resampler.error)

        self._ffi_handle = FfiHandle(resp.new_sox_resampler.resampler.handle.id)

    def push(self, data: bytearray | AudioFrame) -> list[AudioFrame]:
        """
        Push audio data into the resampler and retrieve any available resampled data.

        This method accepts audio data, resamples it according to the configured input and output rates,
        and returns any resampled data that is available after processing the input.

        Args:
            data (bytearray | AudioFrame): The audio data to resample. This can be a `bytearray` containing
                raw audio bytes in int16le format or an `AudioFrame` object.

        Returns:
            list[AudioFrame]: A list of `AudioFrame` objects containing the resampled audio data.
                The list may be empty if no output data is available yet.

        Raises:
            Exception: If there is an error during resampling.
        """
        bdata = data if isinstance(data, bytearray) else data.data.cast("b")

        req = proto_ffi.FfiRequest()
        req.push_sox_resampler.resampler_handle = self._ffi_handle.handle
        req.push_sox_resampler.data_ptr = get_address(memoryview(bdata))
        req.push_sox_resampler.size = len(bdata)

        resp = FfiClient.instance.request(req)

        if resp.push_sox_resampler.error:
            raise Exception(resp.push_sox_resampler.error)

        if not resp.push_sox_resampler.output_ptr:
            return []

        cdata = (ctypes.c_int8 * resp.push_sox_resampler.size).from_address(
            resp.push_sox_resampler.output_ptr
        )
        output_data = bytearray(cdata)
        return [
            AudioFrame(
                output_data,
                self._output_rate,
                self._num_channels,
                len(output_data) // (self._num_channels * ctypes.sizeof(ctypes.c_int16)),
            )
        ]

    def flush(self) -> list[AudioFrame]:
        """
        Flush any remaining audio data through the resampler and retrieve the resampled data.

        This method should be called when no more input data will be provided to ensure that all internal
        buffers are processed and all resampled data is output.

        Returns:
            list[AudioFrame]: A list of `AudioFrame` objects containing the remaining resampled audio data after flushing.
                The list may be empty if no output data remains.

        Raises:
            Exception: If there is an error during flushing.
        """
        req = proto_ffi.FfiRequest()
        req.flush_sox_resampler.resampler_handle = self._ffi_handle.handle

        resp = FfiClient.instance.request(req)

        if not resp.flush_sox_resampler.output_ptr:
            return []

        cdata = (ctypes.c_int8 * resp.flush_sox_resampler.size).from_address(
            resp.flush_sox_resampler.output_ptr
        )
        output_data = bytearray(cdata)
        return [
            AudioFrame(
                output_data,
                self._output_rate,
                self._num_channels,
                len(output_data) // (self._num_channels * ctypes.sizeof(ctypes.c_int16)),
            )
        ]


def _to_proto_quality(
    quality: AudioResamplerQuality,
) -> proto_audio_frame.SoxQualityRecipe.ValueType:
    if quality == AudioResamplerQuality.QUICK:
        return proto_audio_frame.SoxQualityRecipe.SOXR_QUALITY_QUICK
    elif quality == AudioResamplerQuality.LOW:
        return proto_audio_frame.SoxQualityRecipe.SOXR_QUALITY_LOW
    elif quality == AudioResamplerQuality.MEDIUM:
        return proto_audio_frame.SoxQualityRecipe.SOXR_QUALITY_MEDIUM
    elif quality == AudioResamplerQuality.HIGH:
        return proto_audio_frame.SoxQualityRecipe.SOXR_QUALITY_HIGH
    elif quality == AudioResamplerQuality.VERY_HIGH:
        return proto_audio_frame.SoxQualityRecipe.SOXR_QUALITY_VERYHIGH



from __future__ import annotations

import sys
import atexit
import contextlib
import html
from IPython.core.display import HTML
from IPython.display import display
from importlib.resources import as_file, files

_resource_stack = contextlib.ExitStack()
atexit.register(_resource_stack.close)


def room_html(url: str, token: str, *, width: str, height: str) -> HTML:
    """
    Generate the HTML needed to embed a LiveKit room.

    Args:
        url (str): The LiveKit room URL.
        token (str): The LiveKit join token.

    Important:
        The returned HTML contains the provided `url` and `token` values directly.
        Avoid using sensitive tokens in public notebooks (e.g., tokens with long expiration times).
    """
    token_placeholder = "##livekit-token-placeholder##"
    url_placeholder = "##livekit-url-placeholder##"

    index_path = files("livekit.rtc.resources") / "jupyter-html" / "index.html"
    index_path = _resource_stack.enter_context(as_file(index_path))

    # turns out that directly replacing the URL/token is necessary, as Colab or Jupyter comms become
    # unreliable when the main thread is busy/blocked.
    # it also avoid the need to use --expose-app-in-browser when starting jupyter notebook
    html_text = index_path.read_text()
    html_text = html_text.replace(token_placeholder, token)
    html_text = html_text.replace(url_placeholder, url)

    IN_COLAB = "google.colab" in sys.modules

    # Colab output already runs inside an iframe, so we don’t need to create an additional one.
    # It also injects code into this iframe to handle microphone usage, so we need to preserve it.
    if not IN_COLAB:
        escaped_content = html.escape(html_text, quote=True)
        # the extra space in the iframe_html is to avoid IPython suggesting to use IFrame instead.
        # it isn't possible in our case, as we need to use srcdoc.
        html_text = (
            f' <iframe width="{width}" height="{height}" frameborder="0" '
            f'srcdoc="{escaped_content}"></iframe>'
        )

    return HTML(html_text)


def display_room(url: str, token: str, *, width: str = "100%", height: str = "110px") -> None:
    """
    Display a LiveKit room in a Jupyter notebook or Google Colab.

    Args:
        url (str): The LiveKit room URL.
        token (str): The LiveKit join token.

    Important:
        The rendered HTML will include the provided `url` and `token` in plain text.
        Avoid using sensitive tokens in public notebooks (e.g., tokens with long expiration times).
    """
    display(room_html(url, token, width=width, height=height))



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2025 LiveKit, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _VideoQuality:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _VideoQualityEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_VideoQuality.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    VIDEO_QUALITY_LOW: _VideoQuality.ValueType  # 0
    VIDEO_QUALITY_MEDIUM: _VideoQuality.ValueType  # 1
    VIDEO_QUALITY_HIGH: _VideoQuality.ValueType  # 2

class VideoQuality(_VideoQuality, metaclass=_VideoQualityEnumTypeWrapper):
    """Video quality for simulcasted tracks."""

VIDEO_QUALITY_LOW: VideoQuality.ValueType  # 0
VIDEO_QUALITY_MEDIUM: VideoQuality.ValueType  # 1
VIDEO_QUALITY_HIGH: VideoQuality.ValueType  # 2
global___VideoQuality = VideoQuality

@typing.final
class EnableRemoteTrackPublicationRequest(google.protobuf.message.Message):
    """Enable/Disable a remote track publication"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TRACK_PUBLICATION_HANDLE_FIELD_NUMBER: builtins.int
    ENABLED_FIELD_NUMBER: builtins.int
    track_publication_handle: builtins.int
    enabled: builtins.bool
    def __init__(
        self,
        *,
        track_publication_handle: builtins.int | None = ...,
        enabled: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["enabled", b"enabled", "track_publication_handle", b"track_publication_handle"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["enabled", b"enabled", "track_publication_handle", b"track_publication_handle"]) -> None: ...

global___EnableRemoteTrackPublicationRequest = EnableRemoteTrackPublicationRequest

@typing.final
class EnableRemoteTrackPublicationResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___EnableRemoteTrackPublicationResponse = EnableRemoteTrackPublicationResponse

@typing.final
class UpdateRemoteTrackPublicationDimensionRequest(google.protobuf.message.Message):
    """update a remote track publication dimension"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TRACK_PUBLICATION_HANDLE_FIELD_NUMBER: builtins.int
    WIDTH_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    track_publication_handle: builtins.int
    width: builtins.int
    height: builtins.int
    def __init__(
        self,
        *,
        track_publication_handle: builtins.int | None = ...,
        width: builtins.int | None = ...,
        height: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["height", b"height", "track_publication_handle", b"track_publication_handle", "width", b"width"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["height", b"height", "track_publication_handle", b"track_publication_handle", "width", b"width"]) -> None: ...

global___UpdateRemoteTrackPublicationDimensionRequest = UpdateRemoteTrackPublicationDimensionRequest

@typing.final
class UpdateRemoteTrackPublicationDimensionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UpdateRemoteTrackPublicationDimensionResponse = UpdateRemoteTrackPublicationDimensionResponse

@typing.final
class SetRemoteTrackPublicationQualityRequest(google.protobuf.message.Message):
    """For tracks that support simulcasting, adjust subscribed quality."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TRACK_PUBLICATION_HANDLE_FIELD_NUMBER: builtins.int
    QUALITY_FIELD_NUMBER: builtins.int
    track_publication_handle: builtins.int
    quality: global___VideoQuality.ValueType
    def __init__(
        self,
        *,
        track_publication_handle: builtins.int | None = ...,
        quality: global___VideoQuality.ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["quality", b"quality", "track_publication_handle", b"track_publication_handle"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["quality", b"quality", "track_publication_handle", b"track_publication_handle"]) -> None: ...

global___SetRemoteTrackPublicationQualityRequest = SetRemoteTrackPublicationQualityRequest

@typing.final
class SetRemoteTrackPublicationQualityResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___SetRemoteTrackPublicationQualityResponse = SetRemoteTrackPublicationQualityResponse



import os
import wave
import numpy as np

from livekit.rtc import AudioProcessingModule, AudioFrame


def test_audio_processing():
    sample_rate = 48000
    num_channels = 1
    frames_per_chunk = sample_rate // 100

    current_dir = os.path.dirname(os.path.abspath(__file__))
    capture_wav = os.path.join(current_dir, "test_echo_capture.wav")
    render_wav = os.path.join(current_dir, "test_echo_render.wav")
    output_wav = os.path.join(current_dir, "test_processed.wav")

    # Initialize APM with echo cancellation enabled
    apm = AudioProcessingModule(
        echo_cancellation=True,
        noise_suppression=True,
        high_pass_filter=True,
        auto_gain_control=True,
    )
    print("APM Internal Handle:", apm._ffi_handle)

    with (
        wave.open(capture_wav, "rb") as wf_in_cap,
        wave.open(render_wav, "rb") as wf_in_rend,
        wave.open(output_wav, "wb") as wf_out,
    ):
        assert wf_in_cap.getnchannels() == num_channels, "Capture file must be mono."
        assert wf_in_rend.getnchannels() == num_channels, "Render file must be mono."
        assert wf_in_cap.getframerate() == sample_rate, "Capture file must be 48 kHz."
        assert wf_in_rend.getframerate() == sample_rate, "Render file must be 48 kHz."

        sampwidth = wf_in_cap.getsampwidth()
        wf_out.setnchannels(num_channels)
        wf_out.setsampwidth(sampwidth)
        wf_out.setframerate(sample_rate)

        while True:
            capture_bytes = wf_in_cap.readframes(frames_per_chunk)
            render_bytes = wf_in_rend.readframes(frames_per_chunk)

            if not capture_bytes and not render_bytes:
                break

            # Convert bytes to numpy arrays
            capture_data = np.frombuffer(capture_bytes, dtype=np.int16)
            render_data = np.frombuffer(render_bytes, dtype=np.int16)

            # Pad if necessary
            if len(capture_data) < frames_per_chunk:
                capture_data = np.pad(capture_data, (0, frames_per_chunk - len(capture_data)))
            if len(render_data) < frames_per_chunk:
                render_data = np.pad(render_data, (0, frames_per_chunk - len(render_data)))

            capture_frame = AudioFrame(
                data=capture_data.tobytes(),
                sample_rate=sample_rate,
                num_channels=num_channels,
                samples_per_channel=frames_per_chunk,
            )
            render_frame = AudioFrame(
                data=render_data.tobytes(),
                sample_rate=sample_rate,
                num_channels=num_channels,
                samples_per_channel=frames_per_chunk,
            )

            # Process both streams
            apm.process_reverse_stream(render_frame)
            apm.process_stream(capture_frame)

            wf_out.writeframes(capture_frame.data.tobytes())

    print("Done! Processed audio saved to:", output_wav)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: track_publication.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17track_publication.proto\x12\rlivekit.proto\"X\n#EnableRemoteTrackPublicationRequest\x12 \n\x18track_publication_handle\x18\x01 \x02(\x04\x12\x0f\n\x07\x65nabled\x18\x02 \x02(\x08\"&\n$EnableRemoteTrackPublicationResponse\"o\n,UpdateRemoteTrackPublicationDimensionRequest\x12 \n\x18track_publication_handle\x18\x01 \x02(\x04\x12\r\n\x05width\x18\x02 \x02(\r\x12\x0e\n\x06height\x18\x03 \x02(\r\"/\n-UpdateRemoteTrackPublicationDimensionResponse\"y\n\'SetRemoteTrackPublicationQualityRequest\x12 \n\x18track_publication_handle\x18\x01 \x02(\x04\x12,\n\x07quality\x18\x02 \x02(\x0e\x32\x1b.livekit.proto.VideoQuality\"*\n(SetRemoteTrackPublicationQualityResponse*W\n\x0cVideoQuality\x12\x15\n\x11VIDEO_QUALITY_LOW\x10\x00\x12\x18\n\x14VIDEO_QUALITY_MEDIUM\x10\x01\x12\x16\n\x12VIDEO_QUALITY_HIGH\x10\x02\x42\x10\xaa\x02\rLiveKit.Proto')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'track_publication_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\252\002\rLiveKit.Proto'
  _globals['_VIDEOQUALITY']._serialized_start=501
  _globals['_VIDEOQUALITY']._serialized_end=588
  _globals['_ENABLEREMOTETRACKPUBLICATIONREQUEST']._serialized_start=42
  _globals['_ENABLEREMOTETRACKPUBLICATIONREQUEST']._serialized_end=130
  _globals['_ENABLEREMOTETRACKPUBLICATIONRESPONSE']._serialized_start=132
  _globals['_ENABLEREMOTETRACKPUBLICATIONRESPONSE']._serialized_end=170
  _globals['_UPDATEREMOTETRACKPUBLICATIONDIMENSIONREQUEST']._serialized_start=172
  _globals['_UPDATEREMOTETRACKPUBLICATIONDIMENSIONREQUEST']._serialized_end=283
  _globals['_UPDATEREMOTETRACKPUBLICATIONDIMENSIONRESPONSE']._serialized_start=285
  _globals['_UPDATEREMOTETRACKPUBLICATIONDIMENSIONRESPONSE']._serialized_end=332
  _globals['_SETREMOTETRACKPUBLICATIONQUALITYREQUEST']._serialized_start=334
  _globals['_SETREMOTETRACKPUBLICATIONQUALITYREQUEST']._serialized_end=455
  _globals['_SETREMOTETRACKPUBLICATIONQUALITYRESPONSE']._serialized_start=457
  _globals['_SETREMOTETRACKPUBLICATIONQUALITYRESPONSE']._serialized_end=499
# @@protoc_insertion_point(module_scope)
